@@ -22,11 +22,17 @@ from kmip.pie import objects
 from kmip.pie.client import ProxyKmipClient as KmipClient
 
 # Dynamically set __version__
-version_path = os.path.join(os.path.dirname(
-    os.path.realpath(__file__)), 'version.py')
-with open(version_path, 'r') as version_file:
-    mo = re.search(r"^.*= '(\d\.\d\.\d)'$", version_file.read(), re.MULTILINE)
-    __version__ = mo.group(1)
+version_path = os.path.join(
+    os.path.dirname(os.path.realpath(__file__)),
+    "version.py"
+)
+with open(version_path, 'r') as f:
+    m = re.search(
+        r"^__version__ = \"(\d+\.\d+\..*)\"$",
+        f.read(),
+        re.MULTILINE
+    )
+    __version__ = m.group(1)
 
 
 __all__ = [
