@@ -13,6 +13,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import six
+
+from kmip.core import enums
 from kmip.core.enums import Tags
 
 from kmip.core.messages import contents
@@ -22,6 +25,7 @@ from kmip.core.messages.contents import BatchErrorContinuationOption
 from kmip.core.factories.payloads.request import RequestPayloadFactory
 from kmip.core.factories.payloads.response import ResponsePayloadFactory
 
+from kmip.core import primitives
 from kmip.core.primitives import Struct
 
 from kmip.core.utils import BytearrayStream
@@ -48,70 +52,95 @@ class RequestHeader(Struct):
         self.time_stamp = time_stamp
         self.batch_count = batch_count
 
-    def read(self, istream):
-        super(RequestHeader, self).read(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(RequestHeader, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
         tstream = BytearrayStream(istream.read(self.length))
 
         self.protocol_version = contents.ProtocolVersion()
-        self.protocol_version.read(tstream)
+        self.protocol_version.read(tstream, kmip_version=kmip_version)
+
+        kmip_version = contents.protocol_version_to_kmip_version(
+            self.protocol_version
+        )
 
         # Read the maximum response size if it is present
         if self.is_tag_next(Tags.MAXIMUM_RESPONSE_SIZE, tstream):
             self.maximum_response_size = contents.MaximumResponseSize()
-            self.maximum_response_size.read(tstream)
+            self.maximum_response_size.read(tstream, kmip_version=kmip_version)
 
         # Read the asynchronous indicator if it is present
         if self.is_tag_next(Tags.ASYNCHRONOUS_INDICATOR, tstream):
             self.asynchronous_indicator = contents.AsynchronousIndicator()
-            self.asynchronous_indicator.read(tstream)
+            self.asynchronous_indicator.read(
+                tstream,
+                kmip_version=kmip_version
+            )
 
         # Read the authentication if it is present
         if self.is_tag_next(Tags.AUTHENTICATION, tstream):
             self.authentication = contents.Authentication()
-            self.authentication.read(tstream)
+            self.authentication.read(tstream, kmip_version=kmip_version)
 
         # Read the batch error continuation option if it is present
         if self.is_tag_next(Tags.BATCH_ERROR_CONTINUATION_OPTION, tstream):
             self.batch_error_cont_option = BatchErrorContinuationOption()
-            self.batch_error_cont_option.read(tstream)
+            self.batch_error_cont_option.read(
+                tstream,
+                kmip_version=kmip_version
+            )
 
         # Read the batch order option if it is present
         if self.is_tag_next(Tags.BATCH_ORDER_OPTION, tstream):
             self.batch_order_option = contents.BatchOrderOption()
-            self.batch_order_option.read(tstream)
+            self.batch_order_option.read(tstream, kmip_version=kmip_version)
 
         # Read the time stamp if it is present
         if self.is_tag_next(Tags.TIME_STAMP, tstream):
             self.time_stamp = contents.TimeStamp()
-            self.time_stamp.read(tstream)
+            self.time_stamp.read(tstream, kmip_version=kmip_version)
 
         self.batch_count = contents.BatchCount()
-        self.batch_count.read(tstream)
+        self.batch_count.read(tstream, kmip_version=kmip_version)
 
         self.is_oversized(tstream)
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         tstream = BytearrayStream()
 
         # Write the contents of a request header to the stream
-        self.protocol_version.write(tstream)
+        self.protocol_version.write(tstream, kmip_version=kmip_version)
         if self.maximum_response_size is not None:
-            self.maximum_response_size.write(tstream)
+            self.maximum_response_size.write(
+                tstream,
+                kmip_version=kmip_version
+            )
         if self.asynchronous_indicator is not None:
-            self.asynchronous_indicator.write(tstream)
+            self.asynchronous_indicator.write(
+                tstream,
+                kmip_version=kmip_version
+            )
         if self.authentication is not None:
-            self.authentication.write(tstream)
+            self.authentication.write(tstream, kmip_version=kmip_version)
         if self.batch_error_cont_option is not None:
-            self.batch_error_cont_option.write(tstream)
+            self.batch_error_cont_option.write(
+                tstream,
+                kmip_version=kmip_version
+            )
         if self.batch_order_option is not None:
-            self.batch_order_option.write(tstream)
+            self.batch_order_option.write(tstream, kmip_version=kmip_version)
         if self.time_stamp is not None:
-            self.time_stamp.write(tstream)
-        self.batch_count.write(tstream)
+            self.time_stamp.write(tstream, kmip_version=kmip_version)
+        self.batch_count.write(tstream, kmip_version=kmip_version)
 
         # Write the length and value of the request header
         self.length = tstream.length()
-        super(RequestHeader, self).write(ostream)
+        super(RequestHeader, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
 
@@ -120,46 +149,92 @@ class ResponseHeader(Struct):
     def __init__(self,
                  protocol_version=None,
                  time_stamp=None,
-                 batch_count=None):
+                 batch_count=None,
+                 server_hashed_password=None):
         super(ResponseHeader, self).__init__(tag=Tags.RESPONSE_HEADER)
         self.protocol_version = protocol_version
         self.time_stamp = time_stamp
         self.batch_count = batch_count
+        self.server_hashed_password = server_hashed_password
+
         self.validate()
 
-    def read(self, istream):
-        super(ResponseHeader, self).read(istream)
+    @property
+    def server_hashed_password(self):
+        if self._server_hashed_password:
+            return self._server_hashed_password.value
+        return None
+
+    @server_hashed_password.setter
+    def server_hashed_password(self, value):
+        if value is None:
+            self._server_hashed_password = None
+        elif isinstance(value, six.binary_type):
+            self._server_hashed_password = primitives.ByteString(
+                value=value,
+                tag=enums.Tags.SERVER_HASHED_PASSWORD
+            )
+        else:
+            raise TypeError(
+                "The server hashed password must be a binary string."
+            )
+
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(ResponseHeader, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
         tstream = BytearrayStream(istream.read(self.length))
 
         self.protocol_version = contents.ProtocolVersion()
-        self.protocol_version.read(tstream)
+        self.protocol_version.read(tstream, kmip_version=kmip_version)
+
+        kmip_version = contents.protocol_version_to_kmip_version(
+            self.protocol_version
+        )
 
         self.time_stamp = contents.TimeStamp()
-        self.time_stamp.read(tstream)
+        self.time_stamp.read(tstream, kmip_version=kmip_version)
+
+        if kmip_version >= enums.KMIPVersion.KMIP_2_0:
+            if self.is_tag_next(enums.Tags.SERVER_HASHED_PASSWORD, tstream):
+                server_hashed_password = primitives.ByteString(
+                    tag=enums.Tags.SERVER_HASHED_PASSWORD
+                )
+                server_hashed_password.read(tstream, kmip_version=kmip_version)
+                self._server_hashed_password = server_hashed_password
 
         self.batch_count = contents.BatchCount()
-        self.batch_count.read(tstream)
+        self.batch_count.read(tstream, kmip_version=kmip_version)
 
         self.is_oversized(tstream)
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         tstream = BytearrayStream()
 
         # Write the contents of a response header to the stream
-        self.protocol_version.write(tstream)
-        self.time_stamp.write(tstream)
-        self.batch_count.write(tstream)
+        self.protocol_version.write(tstream, kmip_version=kmip_version)
+        self.time_stamp.write(tstream, kmip_version=kmip_version)
+
+        if kmip_version >= enums.KMIPVersion.KMIP_2_0:
+            if self._server_hashed_password:
+                self._server_hashed_password.write(
+                    tstream,
+                    kmip_version=kmip_version
+                )
+
+        self.batch_count.write(tstream, kmip_version=kmip_version)
 
         # Write the length and value of the request header
         self.length = tstream.length()
-        super(ResponseHeader, self).write(ostream)
+        super(ResponseHeader, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
     def validate(self):
-        if self.protocol_version is not None:
-            # TODO (peter-hamilton) conduct type check
-            self.protocol_version.validate()
         if self.time_stamp is not None:
             # TODO (peter-hamilton) conduct type check
             self.time_stamp.validate()
@@ -174,7 +249,8 @@ class RequestBatchItem(Struct):
                  operation=None,
                  unique_batch_item_id=None,
                  request_payload=None,
-                 message_extension=None):
+                 message_extension=None,
+                 ephemeral=None):
         super(RequestBatchItem, self).__init__(tag=Tags.REQUEST_BATCH_ITEM)
 
         self.payload_factory = RequestPayloadFactory()
@@ -183,50 +259,86 @@ class RequestBatchItem(Struct):
         self.unique_batch_item_id = unique_batch_item_id
         self.request_payload = request_payload
         self.message_extension = message_extension
+        self.ephemeral = ephemeral
 
-    def read(self, istream):
-        super(RequestBatchItem, self).read(istream)
+    @property
+    def ephemeral(self):
+        if self._ephemeral:
+            return self._ephemeral.value
+        return None
+
+    @ephemeral.setter
+    def ephemeral(self, value):
+        if value is None:
+            self._ephemeral = None
+        elif isinstance(value, bool):
+            ephemeral = primitives.Boolean(
+                value=value,
+                tag=enums.Tags.EPHEMERAL
+            )
+            self._ephemeral = ephemeral
+        else:
+            raise TypeError("The ephemeral value must be a boolean.")
+
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(RequestBatchItem, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
         tstream = BytearrayStream(istream.read(self.length))
 
         # Read the batch item operation
         self.operation = contents.Operation()
-        self.operation.read(tstream)
+        self.operation.read(tstream, kmip_version=kmip_version)
+
+        if kmip_version >= enums.KMIPVersion.KMIP_2_0:
+            if self.is_tag_next(enums.Tags.EPHEMERAL, tstream):
+                ephemeral = primitives.Boolean(tag=enums.Tags.EPHEMERAL)
+                ephemeral.read(tstream, kmip_version=kmip_version)
+                self._ephemeral = ephemeral
 
         # Read the unique batch item ID if it is present
         if self.is_tag_next(Tags.UNIQUE_BATCH_ITEM_ID, tstream):
             self.unique_batch_item_id = contents.UniqueBatchItemID()
-            self.unique_batch_item_id.read(tstream)
+            self.unique_batch_item_id.read(tstream, kmip_version=kmip_version)
 
         # Dynamically create the response payload class that belongs to the
         # operation
         self.request_payload = self.payload_factory.create(
             self.operation.value)
-        self.request_payload.read(tstream)
+        self.request_payload.read(tstream, kmip_version=kmip_version)
 
         # Read the message extension if it is present
         if self.is_tag_next(Tags.MESSAGE_EXTENSION, tstream):
             self.message_extension = contents.MessageExtension()
-            self.message_extension.read(tstream)
+            self.message_extension.read(tstream, kmip_version=kmip_version)
 
         self.is_oversized(tstream)
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         tstream = BytearrayStream()
 
         # Write the contents of the batch item to the stream
-        self.operation.write(tstream)
+        self.operation.write(tstream, kmip_version=kmip_version)
+
+        if kmip_version >= enums.KMIPVersion.KMIP_2_0:
+            if self._ephemeral:
+                self._ephemeral.write(tstream, kmip_version=kmip_version)
 
         if self.unique_batch_item_id is not None:
-            self.unique_batch_item_id.write(tstream)
+            self.unique_batch_item_id.write(tstream, kmip_version=kmip_version)
 
-        self.request_payload.write(tstream)
+        self.request_payload.write(tstream, kmip_version=kmip_version)
 
         if self.message_extension is not None:
-            self.message_extension.write(tstream)
+            self.message_extension.write(tstream, kmip_version=kmip_version)
 
         # Write the length and value of the batch item
         self.length = tstream.length()
-        super(RequestBatchItem, self).write(ostream)
+        super(RequestBatchItem, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
 
@@ -255,38 +367,44 @@ class ResponseBatchItem(Struct):
         self.message_extension = message_extension
         self.validate()
 
-    def read(self, istream):
-        super(ResponseBatchItem, self).read(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(ResponseBatchItem, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
         tstream = BytearrayStream(istream.read(self.length))
 
         # Read the batch item operation if it is present
         if self.is_tag_next(Tags.OPERATION, tstream):
             self.operation = contents.Operation()
-            self.operation.read(tstream)
+            self.operation.read(tstream, kmip_version=kmip_version)
 
         # Read the unique batch item ID if it is present
         if self.is_tag_next(Tags.UNIQUE_BATCH_ITEM_ID, tstream):
             self.unique_batch_item_id = contents.UniqueBatchItemID()
-            self.unique_batch_item_id.read(tstream)
+            self.unique_batch_item_id.read(tstream, kmip_version=kmip_version)
 
         # Read the batch item result status
         self.result_status = contents.ResultStatus()
-        self.result_status.read(tstream)
+        self.result_status.read(tstream, kmip_version=kmip_version)
 
         # Read the batch item result reason if it is present
         if self.is_tag_next(Tags.RESULT_REASON, tstream):
             self.result_reason = contents.ResultReason()
-            self.result_reason.read(tstream)
+            self.result_reason.read(tstream, kmip_version=kmip_version)
 
         # Read the batch item result message if it is present
         if self.is_tag_next(Tags.RESULT_MESSAGE, tstream):
             self.result_message = contents.ResultMessage()
-            self.result_message.read(tstream)
+            self.result_message.read(tstream, kmip_version=kmip_version)
 
         # Read the batch item asynchronous correlation value if it is present
         if self.is_tag_next(Tags.ASYNCHRONOUS_CORRELATION_VALUE, tstream):
             self.async_correlation_value = AsynchronousCorrelationValue()
-            self.async_correlation_value.read(tstream)
+            self.async_correlation_value.read(
+                tstream,
+                kmip_version=kmip_version
+            )
 
         if (self.operation is not None):
             # Dynamically create the response payload class that belongs to the
@@ -294,41 +412,47 @@ class ResponseBatchItem(Struct):
             expected = self.payload_factory.create(self.operation.value)
             if self.is_tag_next(expected.tag, tstream):
                 self.response_payload = expected
-                self.response_payload.read(tstream)
+                self.response_payload.read(tstream, kmip_version=kmip_version)
 
         # Read the message extension if it is present
         if self.is_tag_next(Tags.MESSAGE_EXTENSION, tstream):
             self.message_extension = contents.MessageExtension()
-            self.message_extension.read(tstream)
+            self.message_extension.read(tstream, kmip_version=kmip_version)
 
         self.is_oversized(tstream)
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         tstream = BytearrayStream()
 
         # Write the contents of the batch item to the stream
         if self.operation is not None:
-            self.operation.write(tstream)
+            self.operation.write(tstream, kmip_version=kmip_version)
         if self.unique_batch_item_id is not None:
-            self.unique_batch_item_id.write(tstream)
+            self.unique_batch_item_id.write(tstream, kmip_version=kmip_version)
 
-        self.result_status.write(tstream)
+        self.result_status.write(tstream, kmip_version=kmip_version)
 
         if self.result_reason is not None:
-            self.result_reason.write(tstream)
+            self.result_reason.write(tstream, kmip_version=kmip_version)
         if self.result_message is not None:
-            self.result_message.write(tstream)
+            self.result_message.write(tstream, kmip_version=kmip_version)
         if self.async_correlation_value is not None:
-            self.async_correlation_value.write(tstream)
+            self.async_correlation_value.write(
+                tstream,
+                kmip_version=kmip_version
+            )
         if self.response_payload is not None:
-            self.response_payload.write(tstream)
+            self.response_payload.write(tstream, kmip_version=kmip_version)
         if self.message_extension is not None:
-            self.message_extension.write(tstream)
+            self.message_extension.write(tstream, kmip_version=kmip_version)
 
         # Write the length and value of the batch item
         self.length = tstream.length()
-        super(ResponseBatchItem, self).write(ostream)
+        super(ResponseBatchItem, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -342,29 +466,39 @@ class RequestMessage(Struct):
         self.request_header = request_header
         self.batch_items = batch_items
 
-    def read(self, istream):
-        super(RequestMessage, self).read(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(RequestMessage, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
 
         self.request_header = RequestHeader()
-        self.request_header.read(istream)
+        self.request_header.read(istream, kmip_version=kmip_version)
+
+        kmip_version = contents.protocol_version_to_kmip_version(
+            self.request_header.protocol_version
+        )
 
         self.batch_items = []
         for _ in range(self.request_header.batch_count.value):
             batch_item = RequestBatchItem()
-            batch_item.read(istream)
+            batch_item.read(istream, kmip_version=kmip_version)
             self.batch_items.append(batch_item)
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         tstream = BytearrayStream()
 
         # Write the request header and all batch items
-        self.request_header.write(tstream)
+        self.request_header.write(tstream, kmip_version=kmip_version)
         for batch_item in self.batch_items:
-            batch_item.write(tstream)
+            batch_item.write(tstream, kmip_version=kmip_version)
 
         # Write the TTLV encoding of the request message
         self.length = tstream.length()
-        super(RequestMessage, self).write(ostream)
+        super(RequestMessage, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
 
@@ -376,30 +510,40 @@ class ResponseMessage(Struct):
         self.batch_items = batch_items
         self.validate()
 
-    def read(self, istream):
-        super(ResponseMessage, self).read(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(ResponseMessage, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
 
         self.response_header = ResponseHeader()
-        self.response_header.read(istream)
+        self.response_header.read(istream, kmip_version=kmip_version)
+
+        kmip_version = contents.protocol_version_to_kmip_version(
+            self.response_header.protocol_version
+        )
 
         self.batch_items = []
         for _ in range(self.response_header.batch_count.value):
             batch_item = ResponseBatchItem()
-            batch_item.read(istream)
+            batch_item.read(istream, kmip_version=kmip_version)
             self.batch_items.append(batch_item)
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         tstream = BytearrayStream()
 
         # Write the request header and all batch items
-        self.response_header.write(tstream)
+        self.response_header.write(tstream, kmip_version=kmip_version)
         for batch_item in self.batch_items:
-            batch_item.write(tstream)
+            batch_item.write(tstream, kmip_version=kmip_version)
 
         # Write the TTLV encoding of the request message
         self.length = tstream.length()
-        super(ResponseMessage, self).write(ostream)
+        super(ResponseMessage, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
     def validate(self):
