@@ -16,6 +16,8 @@
 import six
 
 from kmip.core import enums
+from kmip.core import exceptions
+from kmip.core import objects
 from kmip.core import primitives
 from kmip.core import utils
 
@@ -66,47 +68,67 @@ class GetAttributeListRequestPayload(primitives.Struct):
                 tag=enums.Tags.UNIQUE_IDENTIFIER
             )
         else:
-            raise TypeError("unique identifier must be a string")
+            raise TypeError("Unique identifier must be a string.")
 
-    def read(self, istream):
+    def read(self, input_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the data encoding the GetAttributeList request payload and decode
         it into its constituent parts.
 
         Args:
-            istream (stream): A data stream containing encoded object data,
-                supporting a read method; usually a BytearrayStream object.
+            input_buffer (stream): A data stream containing encoded object
+                data, supporting a read method; usually a BytearrayStream
+                object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(GetAttributeListRequestPayload, self).read(istream)
-        tstream = utils.BytearrayStream(istream.read(self.length))
+        super(GetAttributeListRequestPayload, self).read(
+            input_buffer,
+            kmip_version=kmip_version
+        )
+        local_buffer = utils.BytearrayStream(input_buffer.read(self.length))
 
-        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, tstream):
+        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_buffer):
             self._unique_identifier = primitives.TextString(
                 tag=enums.Tags.UNIQUE_IDENTIFIER
             )
-            self._unique_identifier.read(tstream)
+            self._unique_identifier.read(
+                local_buffer,
+                kmip_version=kmip_version
+            )
         else:
             self._unique_identifier = None
 
-        self.is_oversized(tstream)
+        self.is_oversized(local_buffer)
 
-    def write(self, ostream):
+    def write(self, output_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the data encoding the GetAttributeList request payload to a
         stream.
 
         Args:
-            ostream (stream): A data stream in which to encode object data,
-                supporting a write method; usually a BytearrayStream object.
+            output_buffer (stream): A data stream in which to encode object
+                data, supporting a write method; usually a BytearrayStream
+                object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
-        tstream = utils.BytearrayStream()
+        local_buffer = utils.BytearrayStream()
 
         if self._unique_identifier:
-            self._unique_identifier.write(tstream)
+            self._unique_identifier.write(
+                local_buffer,
+                kmip_version=kmip_version
+            )
 
-        self.length = tstream.length()
-        super(GetAttributeListRequestPayload, self).write(ostream)
-        ostream.write(tstream.buffer)
+        self.length = local_buffer.length()
+        super(GetAttributeListRequestPayload, self).write(
+            output_buffer,
+            kmip_version=kmip_version
+        )
+        output_buffer.write(local_buffer.buffer)
 
     def __repr__(self):
         uid = "unique_identifier={0}".format(self.unique_identifier)
@@ -187,7 +209,7 @@ class GetAttributeListResponsePayload(primitives.Struct):
                 tag=enums.Tags.UNIQUE_IDENTIFIER
             )
         else:
-            raise TypeError("unique identifier must be a string")
+            raise TypeError("Unique identifier must be a string.")
 
     @property
     def attribute_names(self):
@@ -209,7 +231,7 @@ class GetAttributeListResponsePayload(primitives.Struct):
                 name = value[i]
                 if not isinstance(name, six.string_types):
                     raise TypeError(
-                        "attribute_names must be a list of strings; "
+                        "Attribute names must be a list of strings; "
                         "item {0} has type {1}".format(i + 1, type(name))
                     )
                 if name not in names:
@@ -223,57 +245,162 @@ class GetAttributeListResponsePayload(primitives.Struct):
                     )
                 )
         else:
-            raise TypeError("attribute_names must be a list of strings")
+            raise TypeError("Attribute names must be a list of strings.")
 
-    def read(self, istream):
+    def read(self, input_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the data encoding the GetAttributeList response payload and
         decode it into its constituent parts.
 
         Args:
-            istream (stream): A data stream containing encoded object data,
-                supporting a read method; usually a BytearrayStream object.
-        """
-        super(GetAttributeListResponsePayload, self).read(istream)
-        tstream = utils.BytearrayStream(istream.read(self.length))
+            input_buffer (stream): A data stream containing encoded object
+                data, supporting a read method; usually a BytearrayStream
+                object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
 
-        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, tstream):
+        Raises:
+            InvalidKmipEncoding: Raised if the unique identifier or attribute
+                names are missing from the encoded payload.
+        """
+        super(GetAttributeListResponsePayload, self).read(
+            input_buffer,
+            kmip_version=kmip_version
+        )
+        local_buffer = utils.BytearrayStream(input_buffer.read(self.length))
+
+        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_buffer):
             self._unique_identifier = primitives.TextString(
                 tag=enums.Tags.UNIQUE_IDENTIFIER
             )
-            self._unique_identifier.read(tstream)
+            self._unique_identifier.read(
+                local_buffer,
+                kmip_version=kmip_version
+            )
         else:
-            self._unique_identifier = None
+            raise exceptions.InvalidKmipEncoding(
+                "The GetAttributeList response payload encoding is missing "
+                "the unique identifier."
+            )
 
         names = list()
-        while self.is_tag_next(enums.Tags.ATTRIBUTE_NAME, tstream):
-            name = primitives.TextString(tag=enums.Tags.ATTRIBUTE_NAME)
-            name.read(tstream)
-            names.append(name)
-        self._attribute_names = names
+        if kmip_version < enums.KMIPVersion.KMIP_2_0:
+            while self.is_tag_next(enums.Tags.ATTRIBUTE_NAME, local_buffer):
+                name = primitives.TextString(tag=enums.Tags.ATTRIBUTE_NAME)
+                name.read(local_buffer, kmip_version=kmip_version)
+                names.append(name)
+            if len(names) == 0:
+                raise exceptions.InvalidKmipEncoding(
+                    "The GetAttributeList response payload encoding is "
+                    "missing the attribute names."
+                )
+            self._attribute_names = names
+        else:
+            while self.is_tag_next(
+                    enums.Tags.ATTRIBUTE_REFERENCE,
+                    local_buffer
+            ):
+                if self.is_type_next(enums.Types.STRUCTURE, local_buffer):
+                    reference = objects.AttributeReference()
+                    reference.read(local_buffer, kmip_version=kmip_version)
+                    names.append(
+                        primitives.TextString(
+                            value=reference.attribute_name,
+                            tag=enums.Tags.ATTRIBUTE_NAME
+                        )
+                    )
+                elif self.is_type_next(enums.Types.ENUMERATION, local_buffer):
+                    reference = primitives.Enumeration(
+                        enums.Tags,
+                        tag=enums.Tags.ATTRIBUTE_REFERENCE
+                    )
+                    reference.read(local_buffer, kmip_version=kmip_version)
+                    name = enums.convert_attribute_tag_to_name(reference.value)
+                    names.append(
+                        primitives.TextString(
+                            value=name,
+                            tag=enums.Tags.ATTRIBUTE_NAME
+                        )
+                    )
+                else:
+                    raise exceptions.InvalidKmipEncoding(
+                        "The GetAttributeList response payload encoding "
+                        "contains an invalid AttributeReference type."
+                    )
+            self._attribute_names = names
 
-        self.is_oversized(tstream)
+        self.is_oversized(local_buffer)
 
-    def write(self, ostream):
+    def write(self, output_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the data encoding the GetAttributeList response payload to a
         stream.
 
         Args:
-            ostream (stream): A data stream in which to encode object data,
-                supporting a write method; usually a BytearrayStream object.
+            output_buffer (stream): A data stream in which to encode object
+                data, supporting a write method; usually a BytearrayStream
+                object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
+
+        Raises:
+            InvalidField: Raised if the unique identifier or attribute name
+                are not defined.
         """
-        tstream = utils.BytearrayStream()
+        local_buffer = utils.BytearrayStream()
 
         if self._unique_identifier:
-            self._unique_identifier.write(tstream)
+            self._unique_identifier.write(
+                local_buffer,
+                kmip_version=kmip_version
+            )
+        else:
+            raise exceptions.InvalidField(
+                "The GetAttributeList response payload is missing the unique "
+                "identifier field."
+            )
 
-        for attribute_name in self._attribute_names:
-            attribute_name.write(tstream)
+        if self._attribute_names:
+            if kmip_version < enums.KMIPVersion.KMIP_2_0:
+                for attribute_name in self._attribute_names:
+                    attribute_name.write(
+                        local_buffer,
+                        kmip_version=kmip_version
+                    )
+            else:
+                # NOTE (ph) This approach simplifies backwards compatible
+                #           issues but limits easy support for Attribute
+                #           Reference structures going forward, specifically
+                #           limiting the use of VendorIdentification for
+                #           custom attributes. If custom attributes need to
+                #           be retrieved using the GetAttributeList operation
+                #           for KMIP 2.0 applications this code will need to
+                #           change.
+                for attribute_name in self._attribute_names:
+                    t = enums.convert_attribute_name_to_tag(
+                        attribute_name.value
+                    )
+                    e = primitives.Enumeration(
+                        enums.Tags,
+                        value=t,
+                        tag=enums.Tags.ATTRIBUTE_REFERENCE
+                    )
+                    e.write(local_buffer, kmip_version=kmip_version)
 
-        self.length = tstream.length()
-        super(GetAttributeListResponsePayload, self).write(ostream)
-        ostream.write(tstream.buffer)
+        else:
+            raise exceptions.InvalidField(
+                "The GetAttributeList response payload is missing the "
+                "attribute names field."
+            )
+
+        self.length = local_buffer.length()
+        super(GetAttributeListResponsePayload, self).write(
+            output_buffer,
+            kmip_version=kmip_version
+        )
+        output_buffer.write(local_buffer.buffer)
 
     def __repr__(self):
         unique_identifier = "unique_identifier={0}".format(
