@@ -60,52 +60,70 @@ class RevokeRequestPayload(Struct):
             self.revocation_reason = objects.RevocationReason()
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the data encoding the RevokeRequestPayload object and decode it
         into its constituent parts.
         Args:
             istream (Stream): A data stream containing encoded object data,
                 supporting a read method; usually a BytearrayStream object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(RevokeRequestPayload, self).read(istream)
+        super(RevokeRequestPayload, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
         tstream = BytearrayStream(istream.read(self.length))
 
         self.unique_identifier = attributes.UniqueIdentifier()
-        self.unique_identifier.read(tstream)
+        self.unique_identifier.read(tstream, kmip_version=kmip_version)
 
         self.revocation_reason = objects.RevocationReason()
-        self.revocation_reason.read(tstream)
+        self.revocation_reason.read(tstream, kmip_version=kmip_version)
 
         if self.is_tag_next(enums.Tags.COMPROMISE_OCCURRENCE_DATE, tstream):
             self.compromise_occurrence_date = primitives.DateTime(
                 tag=enums.Tags.COMPROMISE_OCCURRENCE_DATE)
-            self.compromise_occurrence_date.read(tstream)
+            self.compromise_occurrence_date.read(
+                tstream,
+                kmip_version=kmip_version
+            )
 
         self.is_oversized(tstream)
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the data encoding the RevokeRequestPayload object to a stream.
         Args:
             ostream (Stream): A data stream in which to encode object data,
                 supporting a write method; usually a BytearrayStream object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
         tstream = BytearrayStream()
 
         # Write the contents of the request payload
         if self.unique_identifier is not None:
-            self.unique_identifier.write(tstream)
+            self.unique_identifier.write(tstream, kmip_version=kmip_version)
 
-        self.revocation_reason.write(tstream)
+        self.revocation_reason.write(tstream, kmip_version=kmip_version)
 
         if self.compromise_occurrence_date is not None:
-            self.compromise_occurrence_date.write(tstream)
+            self.compromise_occurrence_date.write(
+                tstream,
+                kmip_version=kmip_version
+            )
 
         # Write the length and value of the request payload
         self.length = tstream.length()
-        super(RevokeRequestPayload, self).write(ostream)
+        super(RevokeRequestPayload, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -151,38 +169,50 @@ class RevokeResponsePayload(Struct):
             self.unique_identifier = unique_identifier
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the data encoding the RevokeResponsePayload object and decode it
         into its constituent parts.
         Args:
             istream (Stream): A data stream containing encoded object data,
                 supporting a read method; usually a BytearrayStream object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(RevokeResponsePayload, self).read(istream)
+        super(RevokeResponsePayload, self).read(
+            istream,
+            kmip_version=kmip_version
+        )
         tstream = BytearrayStream(istream.read(self.length))
 
         self.unique_identifier = attributes.UniqueIdentifier()
-        self.unique_identifier.read(tstream)
+        self.unique_identifier.read(tstream, kmip_version=kmip_version)
 
         self.is_oversized(tstream)
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the data encoding the RevokeResponsePayload object to a stream.
         Args:
             ostream (Stream): A data stream in which to encode object data,
                 supporting a write method; usually a BytearrayStream object.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
         tstream = BytearrayStream()
 
         # Write the contents of the response payload
-        self.unique_identifier.write(tstream)
+        self.unique_identifier.write(tstream, kmip_version=kmip_version)
 
         # Write the length and value of the request payload
         self.length = tstream.length()
-        super(RevokeResponsePayload, self).write(ostream)
+        super(RevokeResponsePayload, self).write(
+            ostream,
+            kmip_version=kmip_version
+        )
         ostream.write(tstream.buffer)
 
     def validate(self):
