@@ -16,7 +16,7 @@
 from binascii import hexlify
 import io
 
-from kmip.core.errors import ErrorStrings
+from kmip.core import exceptions
 
 
 def bit_length(num):
@@ -52,9 +52,8 @@ def is_stream_empty(stream):
 
 def build_er_error(class_object, descriptor, expected, received,
                    attribute=None):
-    msg = ErrorStrings.BAD_EXP_RECV
+    msg = exceptions.ErrorStrings.BAD_EXP_RECV
 
-    class_string = ''
     if attribute is None:
         class_string = '{0}'.format(class_object.__name__)
     else:
