@@ -88,9 +88,13 @@ if __name__ == '__main__':
     name = 'Demo X.509 Certificate'
 
     cert = objects.X509Certificate(value, usage_mask, name)
+    cert.operation_policy_name = opts.operation_policy_name
 
     # Build the client and connect to the server
-    with client.ProxyKmipClient(config=config) as client:
+    with client.ProxyKmipClient(
+            config=config,
+            config_file=opts.config_file
+    ) as client:
         try:
             uid = client.register(cert)
             logger.info("Successfully registered certificate with ID: "
