@@ -86,6 +86,15 @@ def build_cli_parser(operation=None):
         default="client",
         dest="config",
         help="Client configuration group to load from configuration file")
+    parser.add_option(
+        "-s",
+        "--config-file",
+        action="store",
+        type="str",
+        default=None,
+        dest="config_file",
+        help="Path to the client configuration file."
+    )
 
     if operation is Operation.CREATE:
         parser.add_option(
@@ -104,6 +113,15 @@ def build_cli_parser(operation=None):
             default=None,
             dest="length",
             help="Key length in bits (e.g., 128, 256)")
+        parser.add_option(
+            "-o",
+            "--operation-policy-name",
+            action="store",
+            type="str",
+            default=None,
+            dest="operation_policy_name",
+            help="Operation policy name for the secret (e.g., 'default')"
+        )
     elif operation is Operation.CREATE_KEY_PAIR:
         parser.add_option(
             "-a",
@@ -112,7 +130,7 @@ def build_cli_parser(operation=None):
             type="str",
             default=None,
             dest="algorithm",
-            help="Encryption algorithm for the secret (e.g., AES)")
+            help="Encryption algorithm for the secret (e.g., RSA)")
         parser.add_option(
             "-l",
             "--length",
@@ -120,7 +138,7 @@ def build_cli_parser(operation=None):
             type="int",
             default=None,
             dest="length",
-            help="Key length in bits (e.g., 128, 256)")
+            help="Key length in bits (e.g., 512, 1024, 2048)")
         parser.add_option(
             "-n",
             "--name",
@@ -129,6 +147,16 @@ def build_cli_parser(operation=None):
             default=None,
             dest="name",
             help="Name of key pair to create")
+        parser.add_option(
+            "-o",
+            "--operation-policy-name",
+            action="store",
+            type="str",
+            default=None,
+            dest="operation_policy_name",
+            help="Operation policy name for the secrets (e.g., 'default')"
+        )
+
     elif operation is Operation.DESTROY:
         parser.add_option(
             "-i",
@@ -183,6 +211,24 @@ def build_cli_parser(operation=None):
             default=None,
             dest="uuid",
             help="UID of a managed object")
+    elif operation is Operation.GET_ATTRIBUTES:
+        parser.add_option(
+            "-i",
+            "--uuid",
+            action="store",
+            type="str",
+            default=None,
+            dest="uuid",
+            help="UID of a managed object")
+        parser.add_option(
+            "-a",
+            "--attribute-names",
+            action="append",
+            type="str",
+            default=None,
+            dest="attribute_names",
+            help="List of attribute names to retrieve, defaults to all "
+                 "attributes")
     elif operation is Operation.LOCATE:
         parser.add_option(
             "-n",
@@ -212,6 +258,15 @@ def build_cli_parser(operation=None):
             help=("Type of the object to register. Supported types include: "
                   "CERTIFICATE, PRIVATE_KEY, PUBLIC_KEY, SYMMETRIC_KEY, "
                   "SECRET_DATA"))
+        parser.add_option(
+            "-o",
+            "--operation-policy-name",
+            action="store",
+            type="str",
+            default=None,
+            dest="operation_policy_name",
+            help="Operation policy name for the secret (e.g., 'default')"
+        )
     elif operation is Operation.DISCOVER_VERSIONS:
         parser.add_option(
             "-v",
