@@ -61,7 +61,9 @@ class ProxyKmipClient(object):
                  ssl_version=None,
                  username=None,
                  password=None,
-                 config='client'):
+                 config='client',
+                 config_file=None,
+                 kmip_version=None):
         """
         Construct a ProxyKmipClient.
 
@@ -88,8 +90,14 @@ class ProxyKmipClient(object):
                 file. Use to load a specific set of configuration settings from
                 the configuration file, instead of specifying them manually.
                 Optional, defaults to the default client section, 'client'.
+            config_file (string): The path to the client's configuration file.
+                Optional, defaults to None.
+            kmip_version (KMIPVersion): The KMIP version the client should use
+                when making requests. Optional, defaults to None. If None at
+                request time, the client will use KMIP 1.2.
+
         """
-        self.logger = logging.getLogger()
+        self.logger = logging.getLogger(__name__)
 
         self.attribute_factory = attributes.AttributeFactory()
         self.object_factory = factory.ObjectFactory()
@@ -104,11 +112,48 @@ class ProxyKmipClient(object):
             ssl_version=ssl_version,
             username=username,
             password=password,
-            config=config)
+            config=config,
+            config_file=config_file,
+            kmip_version=kmip_version
+        )
 
         # TODO (peter-hamilton) Add a multiprocessing lock for synchronization.
         self._is_open = False
 
+    @property
+    def kmip_version(self):
+        """
+        Get the KMIP version for the client.
+
+        Return:
+            kmip_version (KMIPVersion): The KMIPVersion enumeration used by
+                the client for KMIP requests.
+        """
+        return self.proxy.kmip_version
+
+    @kmip_version.setter
+    def kmip_version(self, value):
+        """
+        Set the KMIP version for the client.
+
+        Args:
+            value (KMIPVersion): A KMIPVersion enumeration
+
+        Return:
+            None
+
+        Raises:
+            ValueError: if value is not a KMIPVersion enumeration
+
+        Example:
+            >>> client.kmip_version = enums.KMIPVersion.KMIP_1_1
+            >>>
+        """
+        if isinstance(value, enums.KMIPVersion):
+            self.proxy.kmip_version = value
+        else:
+            raise ValueError("KMIP version must be a KMIPVersion enumeration")
+
     def open(self):
         """
         Open the client connection.
@@ -125,8 +170,8 @@ class ProxyKmipClient(object):
                 self.proxy.open()
                 self._is_open = True
             except Exception as e:
-                self.logger.exception("could not open client connection", e)
-                raise e
+                self.logger.error("could not open client connection: %s", e)
+                raise
 
     def close(self):
         """
@@ -142,8 +187,8 @@ class ProxyKmipClient(object):
                 self.proxy.close()
                 self._is_open = False
             except Exception as e:
-                self.logger.exception("could not close client connection", e)
-                raise e
+                self.logger.error("could not close client connection: %s", e)
+                raise
 
     @is_connected
     def create(self, algorithm, length, operation_policy_name=None, name=None,
@@ -202,8 +247,7 @@ class ProxyKmipClient(object):
 
         status = result.result_status.value
         if status == enums.ResultStatus.SUCCESS:
-            uid = result.uuid.value
-            return uid
+            return result.uuid
         else:
             reason = result.result_reason.value
             message = result.result_message.value
@@ -227,13 +271,13 @@ class ProxyKmipClient(object):
             length (int): The length in bits for the key pair.
             operation_policy_name (string): The name of the operation policy
                 to use for the new key pair. Optional, defaults to None.
-            public_name (string): The name to give the public key.
-                                  Optional, defaults to None.
+            public_name (string): The name to give the public key. Optional,
+                defaults to None.
             public_usage_mask (list): A list of CryptographicUsageMask
                 enumerations indicating how the public key should be used.
                 Optional, defaults to None.
-            private_name (string): The name to give the public key.
-                                   Optional, defaults to None.
+            private_name (string): The name to give the public key. Optional,
+                defaults to None.
             private_usage_mask (list): A list of CryptographicUsageMask
                 enumerations indicating how the private key should be used.
                 Optional, defaults to None.
@@ -258,9 +302,21 @@ class ProxyKmipClient(object):
         common_attributes = self._build_common_attributes(
             operation_policy_name
         )
-        key_attributes = self._build_key_attributes(algorithm, length)
-        key_attributes.extend(common_attributes)
-        template = cobjects.CommonTemplateAttribute(attributes=key_attributes)
+
+        algorithm_attribute = self.attribute_factory.create_attribute(
+            enums.AttributeType.CRYPTOGRAPHIC_ALGORITHM,
+            algorithm
+        )
+        length_attribute = self.attribute_factory.create_attribute(
+            enums.AttributeType.CRYPTOGRAPHIC_LENGTH,
+            length
+        )
+
+        common_attributes.extend([algorithm_attribute, length_attribute])
+        template = cobjects.TemplateAttribute(
+            attributes=common_attributes,
+            tag=enums.Tags.COMMON_TEMPLATE_ATTRIBUTE
+        )
 
         # Create public / private specific attributes
         public_template = None
@@ -276,9 +332,10 @@ class ProxyKmipClient(object):
                 )
             ]
         if names or attrs:
-            public_template = cobjects.PublicKeyTemplateAttribute(
+            public_template = cobjects.TemplateAttribute(
                 names=names,
-                attributes=attrs
+                attributes=attrs,
+                tag=enums.Tags.PUBLIC_KEY_TEMPLATE_ATTRIBUTE
             )
 
         private_template = None
@@ -294,9 +351,10 @@ class ProxyKmipClient(object):
                 )
             ]
         if names or attrs:
-            private_template = cobjects.PrivateKeyTemplateAttribute(
+            private_template = cobjects.TemplateAttribute(
                 names=names,
-                attributes=attrs
+                attributes=attrs,
+                tag=enums.Tags.PRIVATE_KEY_TEMPLATE_ATTRIBUTE
             )
 
         # Create the asymmetric key pair and handle the results
@@ -307,8 +365,8 @@ class ProxyKmipClient(object):
 
         status = result.result_status.value
         if status == enums.ResultStatus.SUCCESS:
-            public_uid = result.public_key_uuid.value
-            private_uid = result.private_key_uuid.value
+            public_uid = result.public_key_uuid
+            private_uid = result.private_key_uuid
             return public_uid, private_uid
         else:
             reason = result.result_reason.value
@@ -353,6 +411,14 @@ class ProxyKmipClient(object):
                     managed_object.operation_policy_name
                 )
                 object_attributes.append(opn_attribute)
+        if hasattr(managed_object, 'names'):
+            if managed_object.names:
+                for name in managed_object.names:
+                    name_attribute = self.attribute_factory.create_attribute(
+                        enums.AttributeType.NAME,
+                        name
+                    )
+                    object_attributes.append(name_attribute)
 
         template = cobjects.TemplateAttribute(attributes=object_attributes)
         object_type = managed_object.object_type
@@ -363,13 +429,100 @@ class ProxyKmipClient(object):
 
         status = result.result_status.value
         if status == enums.ResultStatus.SUCCESS:
-            uid = result.uuid.value
-            return uid
+            return result.uuid
         else:
             reason = result.result_reason.value
             message = result.result_message.value
             raise exceptions.KmipOperationFailure(status, reason, message)
 
+    @is_connected
+    def rekey(self,
+              uid=None,
+              offset=None,
+              **kwargs):
+        """
+        Rekey an existing key.
+
+        Args:
+            uid (string): The unique ID of the symmetric key to rekey.
+                Optional, defaults to None.
+            offset (int): The time delta, in seconds, between the new key's
+                initialization date and activation date. Optional, defaults
+                to None.
+            **kwargs (various): A placeholder for object attributes that
+                should be set on the newly rekeyed key. Currently
+                supported attributes include:
+                    activation_date (int)
+                    process_start_date (int)
+                    protect_stop_date (int)
+                    deactivation_date (int)
+
+        Returns:
+            string: The unique ID of the newly rekeyed key.
+
+        Raises:
+            ClientConnectionNotOpen: if the client connection is unusable
+            KmipOperationFailure: if the operation result is a failure
+            TypeError: if the input arguments are invalid
+        """
+        if uid is not None:
+            if not isinstance(uid, six.string_types):
+                raise TypeError("The unique identifier must be a string.")
+        if offset is not None:
+            if not isinstance(offset, six.integer_types):
+                raise TypeError("The offset must be an integer.")
+
+        # TODO (peter-hamilton) Unify attribute handling across operations
+        attributes = []
+        if kwargs.get('activation_date'):
+            attributes.append(
+                self.attribute_factory.create_attribute(
+                    enums.AttributeType.ACTIVATION_DATE,
+                    kwargs.get('activation_date')
+                )
+            )
+        if kwargs.get('process_start_date'):
+            attributes.append(
+                self.attribute_factory.create_attribute(
+                    enums.AttributeType.PROCESS_START_DATE,
+                    kwargs.get('process_start_date')
+                )
+            )
+        if kwargs.get('protect_stop_date'):
+            attributes.append(
+                self.attribute_factory.create_attribute(
+                    enums.AttributeType.PROTECT_STOP_DATE,
+                    kwargs.get('protect_stop_date')
+                )
+            )
+        if kwargs.get('deactivation_date'):
+            attributes.append(
+                self.attribute_factory.create_attribute(
+                    enums.AttributeType.DEACTIVATION_DATE,
+                    kwargs.get('deactivation_date')
+                )
+            )
+        template_attribute = cobjects.TemplateAttribute(
+            attributes=attributes
+        )
+
+        # Derive the new key/data and handle the results
+        result = self.proxy.rekey(
+            uuid=uid,
+            offset=offset,
+            template_attribute=template_attribute
+        )
+
+        status = result.get('result_status')
+        if status == enums.ResultStatus.SUCCESS:
+            return result.get('unique_identifier')
+        else:
+            raise exceptions.KmipOperationFailure(
+                status,
+                result.get('result_reason'),
+                result.get('result_message')
+            )
+
     @is_connected
     def derive_key(self,
                    object_type,
@@ -476,6 +629,13 @@ class ProxyKmipClient(object):
                     kwargs.get('cryptographic_algorithm')
                 )
             )
+        if kwargs.get('cryptographic_usage_mask'):
+            attributes.append(
+                self.attribute_factory.create_attribute(
+                    enums.AttributeType.CRYPTOGRAPHIC_USAGE_MASK,
+                    kwargs.get('cryptographic_usage_mask')
+                )
+            )
         template_attribute = cobjects.TemplateAttribute(
             attributes=attributes
         )
@@ -550,13 +710,69 @@ class ProxyKmipClient(object):
 
         status = result.result_status.value
         if status == enums.ResultStatus.SUCCESS:
-            uids = [uuid.value for uuid in result.uuids]
-            return uids
+            return result.uuids
         else:
             reason = result.result_reason.value
             message = result.result_message.value
             raise exceptions.KmipOperationFailure(status, reason, message)
 
+    @is_connected
+    def check(self,
+              uid=None,
+              usage_limits_count=None,
+              cryptographic_usage_mask=None,
+              lease_time=None):
+        """
+        Check the constraints for a managed object.
+
+        Args:
+            uid (string): The unique ID of the managed object to check.
+                Optional, defaults to None.
+            usage_limits_count (int): The number of items that can be secured
+                with the specified managed object. Optional, defaults to None.
+            cryptographic_usage_mask (list): A list of CryptographicUsageMask
+                enumerations specifying the operations possible with the
+                specified managed object. Optional, defaults to None.
+            lease_time (int): The number of seconds that can be leased for the
+                specified managed object. Optional, defaults to None.
+        """
+        if uid is not None:
+            if not isinstance(uid, six.string_types):
+                raise TypeError("The unique identifier must be a string.")
+        if usage_limits_count is not None:
+            if not isinstance(usage_limits_count, six.integer_types):
+                raise TypeError("The usage limits count must be an integer.")
+        if cryptographic_usage_mask is not None:
+            if not isinstance(cryptographic_usage_mask, list) or \
+                    not all(isinstance(
+                        x,
+                        enums.CryptographicUsageMask
+                    ) for x in cryptographic_usage_mask):
+                raise TypeError(
+                    "The cryptographic usage mask must be a list of "
+                    "CryptographicUsageMask enumerations."
+                )
+        if lease_time is not None:
+            if not isinstance(lease_time, six.integer_types):
+                raise TypeError("The lease time must be an integer.")
+
+        result = self.proxy.check(
+            uid,
+            usage_limits_count,
+            cryptographic_usage_mask,
+            lease_time
+        )
+
+        status = result.get('result_status')
+        if status == enums.ResultStatus.SUCCESS:
+            return result.get('unique_identifier')
+        else:
+            raise exceptions.KmipOperationFailure(
+                status,
+                result.get('result_reason'),
+                result.get('result_message')
+            )
+
     @is_connected
     def get(self, uid=None, key_wrapping_specification=None):
         """
@@ -1235,12 +1451,15 @@ class ProxyKmipClient(object):
                 CryptographicParameters struct.
 
         Returns:
+            None: if value is None
             CryptographicParameters: a CryptographicParameters struct
 
         Raises:
             TypeError: if the input argument is invalid
         """
-        if not isinstance(value, dict):
+        if value is None:
+            return None
+        elif not isinstance(value, dict):
             raise TypeError("Cryptographic parameters must be a dictionary.")
 
         cryptographic_parameters = CryptographicParameters(
