@@ -15,6 +15,7 @@
 
 from abc import abstractmethod
 from sqlalchemy import Column, event, ForeignKey, Integer, String, VARBINARY
+from sqlalchemy import Boolean
 from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.orm import relationship
 
@@ -52,6 +53,13 @@ class ManagedObject(sql.Base):
     _names = relationship('ManagedObjectName', back_populates='mo',
                           cascade='all, delete-orphan')
     names = association_proxy('_names', 'name')
+    operation_policy_name = Column(
+        'operation_policy_name',
+        String(50),
+        default='default'
+    )
+    initial_date = Column(Integer, default=0)
+    _owner = Column('owner', String(50), default=None)
 
     __mapper_args__ = {
         'polymorphic_identity': 'ManagedObject',
@@ -71,19 +79,20 @@ class ManagedObject(sql.Base):
         self.unique_identifier = None
         self.name_index = 0
         self.names = list()
+        self.operation_policy_name = None
+        self.initial_date = 0
         self._object_type = None
+        self._owner = None
 
         # All remaining attributes are not considered part of the public API
         # and are subject to change.
         self._application_specific_informations = list()
         self._contact_information = None
         self._object_groups = list()
-        self._operation_policy_name = None
 
         # The following attributes are placeholders for attributes that are
         # unsupported by kmip.core
         self._archive_date = None
-        self._initial_date = None
         self._last_change_date = None
 
     @property
@@ -152,6 +161,7 @@ class CryptographicObject(ManagedObject):
                                primary_key=True)
     cryptographic_usage_masks = Column('cryptographic_usage_mask',
                                        sql.UsageMaskType)
+    state = Column('state', sql.EnumType(enums.State))
     __mapper_args__ = {
         'polymorphic_identity': 'CryptographicObject'
     }
@@ -168,6 +178,7 @@ class CryptographicObject(ManagedObject):
         super(CryptographicObject, self).__init__()
 
         self.cryptographic_usage_masks = list()
+        self.state = enums.State.PRE_ACTIVE
 
         # All remaining attributes are not considered part of the public API
         # and are subject to change.
@@ -184,7 +195,6 @@ class CryptographicObject(ManagedObject):
         self._lease_time = None
         self._links = list()
         self._revocation_reason = None
-        self._state = None
 
 
 class Key(CryptographicObject):
@@ -203,6 +213,8 @@ class Key(CryptographicObject):
         cryptographic_length: An int defining the length of the key in bits.
         key_format_type: A KeyFormatType enumeration defining the format of
             the key value.
+        key_wrapping_data: A dictionary containing key wrapping data
+            settings, describing how the key value has been wrapped.
     """
 
     __tablename__ = 'keys'
@@ -215,6 +227,167 @@ class Key(CryptographicObject):
     key_format_type = Column(
         'key_format_type', sql.EnumType(enums.KeyFormatType))
 
+    # Key wrapping data fields
+    _kdw_wrapping_method = Column(
+        '_kdw_wrapping_method',
+        sql.EnumType(enums.WrappingMethod),
+        default=None
+    )
+    _kdw_eki_unique_identifier = Column(
+        '_kdw_eki_unique_identifier',
+        String,
+        default=None
+    )
+    _kdw_eki_cp_block_cipher_mode = Column(
+        '_kdw_eki_cp_block_cipher_mode',
+        sql.EnumType(enums.BlockCipherMode),
+        default=None
+    )
+    _kdw_eki_cp_padding_method = Column(
+        '_kdw_eki_cp_padding_method',
+        sql.EnumType(enums.PaddingMethod),
+        default=None
+    )
+    _kdw_eki_cp_hashing_algorithm = Column(
+        '_kdw_eki_cp_hashing_algorithm',
+        sql.EnumType(enums.HashingAlgorithm),
+        default=None
+    )
+    _kdw_eki_cp_key_role_type = Column(
+        '_kdw_eki_cp_key_role_type',
+        sql.EnumType(enums.KeyRoleType),
+        default=None
+    )
+    _kdw_eki_cp_digital_signature_algorithm = Column(
+        '_kdw_eki_cp_digital_signature_algorithm',
+        sql.EnumType(enums.DigitalSignatureAlgorithm),
+        default=None
+    )
+    _kdw_eki_cp_cryptographic_algorithm = Column(
+        '_kdw_eki_cp_cryptographic_algorithm',
+        sql.EnumType(enums.CryptographicAlgorithm),
+        default=None
+    )
+    _kdw_eki_cp_random_iv = Column(
+        '_kdw_eki_cp_random_iv',
+        Boolean,
+        default=None
+    )
+    _kdw_eki_cp_iv_length = Column(
+        '_kdw_eki_cp_iv_length',
+        Integer,
+        default=None
+    )
+    _kdw_eki_cp_tag_length = Column(
+        '_kdw_eki_cp_tag_length',
+        Integer,
+        default=None
+    )
+    _kdw_eki_cp_fixed_field_length = Column(
+        '_kdw_eki_cp_fixed_field_length',
+        Integer,
+        default=None
+    )
+    _kdw_eki_cp_invocation_field_length = Column(
+        '_kdw_eki_cp_invocation_field_length',
+        Integer
+    )
+    _kdw_eki_cp_counter_length = Column(
+        '_kdw_eki_cp_counter_length',
+        Integer,
+        default=None
+    )
+    _kdw_eki_cp_initial_counter_value = Column(
+        '_kdw_eki_cp_initial_counter_value',
+        Integer,
+        default=None
+    )
+    _kdw_mski_unique_identifier = Column(
+        '_kdw_mski_unique_identifier',
+        String,
+        default=None
+    )
+    _kdw_mski_cp_block_cipher_mode = Column(
+        '_kdw_mski_cp_block_cipher_mode',
+        sql.EnumType(enums.BlockCipherMode),
+        default=None
+    )
+    _kdw_mski_cp_padding_method = Column(
+        '_kdw_mski_cp_padding_method',
+        sql.EnumType(enums.PaddingMethod),
+        default=None
+    )
+    _kdw_mski_cp_hashing_algorithm = Column(
+        '_kdw_mski_cp_hashing_algorithm',
+        sql.EnumType(enums.HashingAlgorithm),
+        default=None
+    )
+    _kdw_mski_cp_key_role_type = Column(
+        '_kdw_mski_cp_key_role_type',
+        sql.EnumType(enums.KeyRoleType),
+        default=None
+    )
+    _kdw_mski_cp_digital_signature_algorithm = Column(
+        '_kdw_mski_cp_digital_signature_algorithm',
+        sql.EnumType(enums.DigitalSignatureAlgorithm),
+        default=None
+    )
+    _kdw_mski_cp_cryptographic_algorithm = Column(
+        '_kdw_mski_cp_cryptographic_algorithm',
+        sql.EnumType(enums.CryptographicAlgorithm),
+        default=None
+    )
+    _kdw_mski_cp_random_iv = Column(
+        '_kdw_mski_cp_random_iv',
+        Boolean,
+        default=None
+    )
+    _kdw_mski_cp_iv_length = Column(
+        '_kdw_mski_cp_iv_length',
+        Integer,
+        default=None
+    )
+    _kdw_mski_cp_tag_length = Column(
+        '_kdw_mski_cp_tag_length',
+        Integer,
+        default=None
+    )
+    _kdw_mski_cp_fixed_field_length = Column(
+        '_kdw_mski_cp_fixed_field_length',
+        Integer,
+        default=None
+    )
+    _kdw_mski_cp_invocation_field_length = Column(
+        '_kdw_mski_cp_invocation_field_length',
+        Integer,
+        default=None
+    )
+    _kdw_mski_cp_counter_length = Column(
+        '_kdw_mski_cp_counter_length',
+        Integer,
+        default=None
+    )
+    _kdw_mski_cp_initial_counter_value = Column(
+        '_kdw_mski_cp_initial_counter_value',
+        Integer,
+        default=None
+    )
+    _kdw_mac_signature = Column(
+        '_kdw_mac_signature',
+        VARBINARY(1024),
+        default=None
+    )
+    _kdw_iv_counter_nonce = Column(
+        '_kdw_iv_counter_nonce',
+        VARBINARY(1024),
+        default=None
+    )
+    _kdw_encoding_option = Column(
+        '_kdw_encoding_option',
+        sql.EnumType(enums.EncodingOption),
+        default=None
+    )
+
     __mapper_args__ = {
         'polymorphic_identity': 'Key'
     }
@@ -223,15 +396,21 @@ class Key(CryptographicObject):
     }
 
     @abstractmethod
-    def __init__(self):
+    def __init__(self, key_wrapping_data=None):
         """
         Create a Key object.
+
+        Args:
+            key_wrapping_data(dict): A dictionary containing key wrapping data
+                settings, describing how the key value has been wrapped.
+                Optional, defaults to None.
         """
         super(Key, self).__init__()
 
         self.cryptographic_algorithm = None
         self.cryptographic_length = None
         self.key_format_type = None
+        self.key_wrapping_data = key_wrapping_data
 
         # All remaining attributes are not considered part of the public API
         # and are subject to change.
@@ -241,6 +420,145 @@ class Key(CryptographicObject):
         # unsupported by kmip.core
         self._usage_limits = None
 
+    @property
+    def key_wrapping_data(self):
+        """
+        Retrieve all of the relevant key wrapping data fields and return them
+        as a dictionary.
+        """
+        key_wrapping_data = {}
+        encryption_key_info = {
+            'unique_identifier': self._kdw_eki_unique_identifier,
+            'cryptographic_parameters': {
+                'block_cipher_mode': self._kdw_eki_cp_block_cipher_mode,
+                'padding_method': self._kdw_eki_cp_padding_method,
+                'hashing_algorithm': self._kdw_eki_cp_hashing_algorithm,
+                'key_role_type': self._kdw_eki_cp_key_role_type,
+                'digital_signature_algorithm':
+                    self._kdw_eki_cp_digital_signature_algorithm,
+                'cryptographic_algorithm':
+                    self._kdw_eki_cp_cryptographic_algorithm,
+                'random_iv': self._kdw_eki_cp_random_iv,
+                'iv_length': self._kdw_eki_cp_iv_length,
+                'tag_length': self._kdw_eki_cp_tag_length,
+                'fixed_field_length': self._kdw_eki_cp_fixed_field_length,
+                'invocation_field_length':
+                    self._kdw_eki_cp_invocation_field_length,
+                'counter_length': self._kdw_eki_cp_counter_length,
+                'initial_counter_value':
+                    self._kdw_eki_cp_initial_counter_value
+            }
+        }
+        if not any(encryption_key_info['cryptographic_parameters'].values()):
+            encryption_key_info['cryptographic_parameters'] = {}
+        if not any(encryption_key_info.values()):
+            encryption_key_info = {}
+
+        mac_sign_key_info = {
+            'unique_identifier': self._kdw_mski_unique_identifier,
+            'cryptographic_parameters': {
+                'block_cipher_mode': self._kdw_mski_cp_block_cipher_mode,
+                'padding_method': self._kdw_mski_cp_padding_method,
+                'hashing_algorithm': self._kdw_mski_cp_hashing_algorithm,
+                'key_role_type': self._kdw_mski_cp_key_role_type,
+                'digital_signature_algorithm':
+                    self._kdw_mski_cp_digital_signature_algorithm,
+                'cryptographic_algorithm':
+                    self._kdw_mski_cp_cryptographic_algorithm,
+                'random_iv': self._kdw_mski_cp_random_iv,
+                'iv_length': self._kdw_mski_cp_iv_length,
+                'tag_length': self._kdw_mski_cp_tag_length,
+                'fixed_field_length': self._kdw_mski_cp_fixed_field_length,
+                'invocation_field_length':
+                    self._kdw_mski_cp_invocation_field_length,
+                'counter_length': self._kdw_mski_cp_counter_length,
+                'initial_counter_value':
+                    self._kdw_mski_cp_initial_counter_value
+            }
+        }
+        if not any(mac_sign_key_info['cryptographic_parameters'].values()):
+            mac_sign_key_info['cryptographic_parameters'] = {}
+        if not any(mac_sign_key_info.values()):
+            mac_sign_key_info = {}
+
+        key_wrapping_data['wrapping_method'] = self._kdw_wrapping_method
+        key_wrapping_data['encryption_key_information'] = encryption_key_info
+        key_wrapping_data['mac_signature_key_information'] = mac_sign_key_info
+        key_wrapping_data['mac_signature'] = self._kdw_mac_signature
+        key_wrapping_data['iv_counter_nonce'] = self._kdw_iv_counter_nonce
+        key_wrapping_data['encoding_option'] = self._kdw_encoding_option
+        if not any(key_wrapping_data.values()):
+            key_wrapping_data = {}
+
+        return key_wrapping_data
+
+    @key_wrapping_data.setter
+    def key_wrapping_data(self, value):
+        """
+        Set the key wrapping data attributes using a dictionary.
+        """
+        if value is None:
+            value = {}
+        elif not isinstance(value, dict):
+            raise TypeError("Key wrapping data must be a dictionary.")
+
+        self._kdw_wrapping_method = value.get('wrapping_method')
+
+        eki = value.get('encryption_key_information')
+        if eki is None:
+            eki = {}
+        self._kdw_eki_unique_identifier = eki.get('unique_identifier')
+        eki_cp = eki.get('cryptographic_parameters')
+        if eki_cp is None:
+            eki_cp = {}
+        self._kdw_eki_cp_block_cipher_mode = eki_cp.get('block_cipher_mode')
+        self._kdw_eki_cp_padding_method = eki_cp.get('padding_method')
+        self._kdw_eki_cp_hashing_algorithm = eki_cp.get('hashing_algorithm')
+        self._kdw_eki_cp_key_role_type = eki_cp.get('key_role_type')
+        self._kdw_eki_cp_digital_signature_algorithm = \
+            eki_cp.get('digital_signature_algorithm')
+        self._kdw_eki_cp_cryptographic_algorithm = \
+            eki_cp.get('cryptographic_algorithm')
+        self._kdw_eki_cp_random_iv = eki_cp.get('random_iv')
+        self._kdw_eki_cp_iv_length = eki_cp.get('iv_length')
+        self._kdw_eki_cp_tag_length = eki_cp.get('tag_length')
+        self._kdw_eki_cp_fixed_field_length = eki_cp.get('fixed_field_length')
+        self._kdw_eki_cp_invocation_field_length = \
+            eki_cp.get('invocation_field_length')
+        self._kdw_eki_cp_counter_length = eki_cp.get('counter_length')
+        self._kdw_eki_cp_initial_counter_value = \
+            eki_cp.get('initial_counter_value')
+
+        mski = value.get('mac_signature_key_information')
+        if mski is None:
+            mski = {}
+        self._kdw_mski_unique_identifier = mski.get('unique_identifier')
+        mski_cp = mski.get('cryptographic_parameters')
+        if mski_cp is None:
+            mski_cp = {}
+        self._kdw_mski_cp_block_cipher_mode = mski_cp.get('block_cipher_mode')
+        self._kdw_mski_cp_padding_method = mski_cp.get('padding_method')
+        self._kdw_mski_cp_hashing_algorithm = mski_cp.get('hashing_algorithm')
+        self._kdw_mski_cp_key_role_type = mski_cp.get('key_role_type')
+        self._kdw_mski_cp_digital_signature_algorithm = \
+            mski_cp.get('digital_signature_algorithm')
+        self._kdw_mski_cp_cryptographic_algorithm = \
+            mski_cp.get('cryptographic_algorithm')
+        self._kdw_mski_cp_random_iv = mski_cp.get('random_iv')
+        self._kdw_mski_cp_iv_length = mski_cp.get('iv_length')
+        self._kdw_mski_cp_tag_length = mski_cp.get('tag_length')
+        self._kdw_mski_cp_fixed_field_length = \
+            mski_cp.get('fixed_field_length')
+        self._kdw_mski_cp_invocation_field_length = \
+            mski_cp.get('invocation_field_length')
+        self._kdw_mski_cp_counter_length = mski_cp.get('counter_length')
+        self._kdw_mski_cp_initial_counter_value = \
+            mski_cp.get('initial_counter_value')
+
+        self._kdw_mac_signature = value.get('mac_signature')
+        self._kdw_iv_counter_nonce = value.get('iv_counter_nonce')
+        self._kdw_encoding_option = value.get('encoding_option')
+
 
 class SymmetricKey(Key):
     """
@@ -258,6 +576,8 @@ class SymmetricKey(Key):
         cryptographic_usage_masks: The list of usage mask flags for
             SymmetricKey application.
         names: The string names of the SymmetricKey.
+        key_wrapping_data: A dictionary containing key wrapping data
+            settings, describing how the key value has been wrapped.
     """
 
     __tablename__ = 'symmetric_keys'
@@ -273,7 +593,7 @@ class SymmetricKey(Key):
     }
 
     def __init__(self, algorithm, length, value, masks=None,
-                 name='Symmetric Key'):
+                 name='Symmetric Key', key_wrapping_data=None):
         """
         Create a SymmetricKey.
 
@@ -286,8 +606,13 @@ class SymmetricKey(Key):
                 how the key will be used. Optional, defaults to None.
             name(string): The string name of the key. Optional, defaults to
                 'Symmetric Key'.
+            key_wrapping_data(dict): A dictionary containing key wrapping data
+                settings, describing how the key value has been wrapped.
+                Optional, defaults to None.
         """
-        super(SymmetricKey, self).__init__()
+        super(SymmetricKey, self).__init__(
+            key_wrapping_data=key_wrapping_data
+        )
 
         self._object_type = enums.ObjectType.SYMMETRIC_KEY
         self.key_format_type = enums.KeyFormatType.RAW
@@ -344,17 +669,29 @@ class SymmetricKey(Key):
                 raise TypeError("key name {0} must be a string".format(
                     position))
 
-        if (len(self.value) * 8) != self.cryptographic_length:
-            msg = "key length ({0}) not equal to key value length ({1})"
-            msg = msg.format(self.cryptographic_length, len(self.value) * 8)
-            raise ValueError(msg)
+        if not self.key_wrapping_data:
+            if (len(self.value) * 8) != self.cryptographic_length:
+                msg = "key length ({0}) not equal to key value length ({1})"
+                msg = msg.format(
+                    self.cryptographic_length,
+                    len(self.value) * 8
+                )
+                raise ValueError(msg)
 
     def __repr__(self):
         algorithm = "algorithm={0}".format(self.cryptographic_algorithm)
         length = "length={0}".format(self.cryptographic_length)
         value = "value={0}".format(binascii.hexlify(self.value))
+        key_wrapping_data = "key_wrapping_data={0}".format(
+            self.key_wrapping_data
+        )
 
-        return "SymmetricKey({0}, {1}, {2})".format(algorithm, length, value)
+        return "SymmetricKey({0}, {1}, {2}, {3})".format(
+            algorithm,
+            length,
+            value,
+            key_wrapping_data
+        )
 
     def __str__(self):
         return str(binascii.hexlify(self.value))
@@ -367,6 +704,8 @@ class SymmetricKey(Key):
                 return False
             elif self.cryptographic_length != other.cryptographic_length:
                 return False
+            elif self.key_wrapping_data != other.key_wrapping_data:
+                return False
             else:
                 return True
         else:
@@ -399,6 +738,8 @@ class PublicKey(Key):
         cryptographic_usage_masks: The list of usage mask flags for PublicKey
             application.
         names: The list of string names of the PublicKey.
+        key_wrapping_data(dict): A dictionary containing key wrapping data
+            settings, describing how the key value has been wrapped.
     """
 
     __tablename__ = 'public_keys'
@@ -415,7 +756,7 @@ class PublicKey(Key):
 
     def __init__(self, algorithm, length, value,
                  format_type=enums.KeyFormatType.X_509, masks=None,
-                 name='Public Key'):
+                 name='Public Key', key_wrapping_data=None):
         """
         Create a PublicKey.
 
@@ -430,8 +771,13 @@ class PublicKey(Key):
                 defining how the key will be used. Optional, defaults to None.
             name(string): The string name of the key. Optional, defaults to
                 'Public Key'.
+            key_wrapping_data(dict): A dictionary containing key wrapping data
+                settings, describing how the key value has been wrapped.
+                Optional, defaults to None.
         """
-        super(PublicKey, self).__init__()
+        super(PublicKey, self).__init__(
+            key_wrapping_data=key_wrapping_data
+        )
 
         self._object_type = enums.ObjectType.PUBLIC_KEY
         self._valid_formats = [
@@ -503,9 +849,12 @@ class PublicKey(Key):
         length = "length={0}".format(self.cryptographic_length)
         value = "value={0}".format(binascii.hexlify(self.value))
         format_type = "format_type={0}".format(self.key_format_type)
+        key_wrapping_data = "key_wrapping_data={0}".format(
+            self.key_wrapping_data
+        )
 
-        return "PublicKey({0}, {1}, {2}, {3})".format(
-            algorithm, length, value, format_type)
+        return "PublicKey({0}, {1}, {2}, {3}, {4})".format(
+            algorithm, length, value, format_type, key_wrapping_data)
 
     def __str__(self):
         return str(binascii.hexlify(self.value))
@@ -520,6 +869,8 @@ class PublicKey(Key):
                 return False
             elif self.cryptographic_length != other.cryptographic_length:
                 return False
+            elif self.key_wrapping_data != other.key_wrapping_data:
+                return False
             else:
                 return True
         else:
@@ -553,6 +904,8 @@ class PrivateKey(Key):
             application. Optional, defaults to None.
         names: The list of string names of the PrivateKey. Optional, defaults
             to 'Private Key'.
+        key_wrapping_data(dict): A dictionary containing key wrapping data
+            settings, describing how the key value has been wrapped.
     """
 
     __tablename__ = 'private_keys'
@@ -568,7 +921,7 @@ class PrivateKey(Key):
     }
 
     def __init__(self, algorithm, length, value, format_type, masks=None,
-                 name='Private Key'):
+                 name='Private Key', key_wrapping_data=None):
         """
         Create a PrivateKey.
 
@@ -582,8 +935,13 @@ class PrivateKey(Key):
             masks(list): A list of CryptographicUsageMask enumerations
                 defining how the key will be used.
             name(string): The string name of the key.
+            key_wrapping_data(dict): A dictionary containing key wrapping data
+                settings, describing how the key value has been wrapped.
+                Optional, defaults to None.
         """
-        super(PrivateKey, self).__init__()
+        super(PrivateKey, self).__init__(
+            key_wrapping_data=key_wrapping_data
+        )
 
         self._object_type = enums.ObjectType.PRIVATE_KEY
         self._valid_formats = [
@@ -655,9 +1013,12 @@ class PrivateKey(Key):
         length = "length={0}".format(self.cryptographic_length)
         value = "value={0}".format(binascii.hexlify(self.value))
         format_type = "format_type={0}".format(self.key_format_type)
+        key_wrapping_data = "key_wrapping_data={0}".format(
+            self.key_wrapping_data
+        )
 
-        return "PrivateKey({0}, {1}, {2}, {3})".format(
-            algorithm, length, value, format_type)
+        return "PrivateKey({0}, {1}, {2}, {3}, {4})".format(
+            algorithm, length, value, format_type, key_wrapping_data)
 
     def __str__(self):
         return str(binascii.hexlify(self.value))
@@ -672,6 +1033,8 @@ class PrivateKey(Key):
                 return False
             elif self.cryptographic_length != other.cryptographic_length:
                 return False
+            elif self.key_wrapping_data != other.key_wrapping_data:
+                return False
             else:
                 return True
         else:
@@ -709,7 +1072,7 @@ class Certificate(CryptographicObject):
                                ForeignKey('crypto_objects.uid'),
                                primary_key=True)
     certificate_type = Column(
-        'certificate_type', sql.EnumType(enums.CertificateTypeEnum))
+        'certificate_type', sql.EnumType(enums.CertificateType))
 
     __mapper_args__ = {
         'polymorphic_identity': 'Certificate'
@@ -766,8 +1129,8 @@ class Certificate(CryptographicObject):
         if not isinstance(self.value, bytes):
             raise TypeError("certificate value must be bytes")
         elif not isinstance(self.certificate_type,
-                            enums.CertificateTypeEnum):
-            raise TypeError("certificate type must be a CertificateTypeEnum "
+                            enums.CertificateType):
+            raise TypeError("certificate type must be a CertificateType "
                             "enumeration")
 
         mask_count = len(self.cryptographic_usage_masks)
@@ -812,7 +1175,7 @@ class X509Certificate(Certificate):
                                primary_key=True)
 
     __mapper_args__ = {
-        'polymorphic_identity': 'Certificate'
+        'polymorphic_identity': 'X509Certificate'
     }
     __table_args__ = {
         'sqlite_autoincrement': True
@@ -829,7 +1192,7 @@ class X509Certificate(Certificate):
             name(string): The string name of the certificate.
         """
         super(X509Certificate, self).__init__(
-            enums.CertificateTypeEnum.X_509, value, masks, name)
+            enums.CertificateType.X_509, value, masks, name)
 
         # All remaining attributes are not considered part of the public API
         # and are subject to change.
