@@ -13,6 +13,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from __future__ import print_function
+
 from kmip.services.results import ActivateResult
 from kmip.services.results import CreateResult
 from kmip.services.results import CreateKeyPairResult
@@ -31,6 +33,7 @@ from kmip.services.results import MACResult
 
 from kmip.core import attributes as attr
 
+from kmip.core import enums
 from kmip.core.enums import AuthenticationSuite
 from kmip.core.enums import ConformanceClause
 from kmip.core.enums import CredentialType
@@ -39,7 +42,6 @@ from kmip.core.enums import Operation as OperationEnum
 from kmip.core.factories.credentials import CredentialFactory
 
 from kmip.core import objects
-from kmip.core.server import KMIP
 
 from kmip.core.messages.contents import Authentication
 from kmip.core.messages.contents import BatchCount
@@ -50,7 +52,7 @@ from kmip.core.messages import messages
 
 from kmip.core.messages import payloads
 
-from kmip.services.server.kmip_protocol import KMIPProtocol
+from kmip.services.kmip_protocol import KMIPProtocol
 
 from kmip.core.config_helper import ConfigHelper
 
@@ -59,30 +61,54 @@ from kmip.core.utils import BytearrayStream
 import logging
 import logging.config
 import os
+import six
 import socket
 import ssl
+import sys
 
 FILE_PATH = os.path.dirname(os.path.abspath(__file__))
 CONFIG_FILE = os.path.normpath(os.path.join(FILE_PATH, '../kmipconfig.ini'))
 
 
-class KMIPProxy(KMIP):
+class KMIPProxy(object):
 
     def __init__(self, host=None, port=None, keyfile=None,
                  certfile=None,
                  cert_reqs=None, ssl_version=None, ca_certs=None,
                  do_handshake_on_connect=None,
                  suppress_ragged_eofs=None,
-                 username=None, password=None, timeout=30, config='client'):
-        super(KMIPProxy, self).__init__()
+                 username=None, password=None, timeout=30, config='client',
+                 config_file=None,
+                 kmip_version=None):
         self.logger = logging.getLogger(__name__)
         self.credential_factory = CredentialFactory()
         self.config = config
+        # Even partially-initialized objects need to be garbage collected, so
+        # make sure we have a socket attr before we go raising ValueErrors.
+        # Otherwise, we can hit AttributeErrors when __del__ is called.
+        self.socket = None
+
+        self._kmip_version = None
+        if kmip_version:
+            self.kmip_version = kmip_version
+        else:
+            self.kmip_version = enums.KMIPVersion.KMIP_1_2
+
+        if config_file:
+            if not isinstance(config_file, six.string_types):
+                raise ValueError(
+                    "The client configuration file argument must be a string."
+                )
+            if not os.path.exists(config_file):
+                raise ValueError(
+                    "The client configuration file '{}' does not "
+                    "exist.".format(config_file)
+                )
 
         self._set_variables(host, port, keyfile, certfile,
                             cert_reqs, ssl_version, ca_certs,
                             do_handshake_on_connect, suppress_ragged_eofs,
-                            username, password, timeout)
+                            username, password, timeout, config_file)
         self.batch_items = []
 
         self.conformance_clauses = [
@@ -91,7 +117,40 @@ class KMIPProxy(KMIP):
         self.authentication_suites = [
             AuthenticationSuite.BASIC,
             AuthenticationSuite.TLS12]
-        self.socket = None
+
+    @property
+    def kmip_version(self):
+        """
+        Get the KMIP version for the client.
+
+        Return:
+            kmip_version (KMIPVersion): The KMIPVersion enumeration used by
+                the client for KMIP requests.
+        """
+        return self._kmip_version
+
+    @kmip_version.setter
+    def kmip_version(self, value):
+        """
+        Set the KMIP version for the client.
+
+        Args:
+            value (KMIPVersion): A KMIPVersion enumeration
+
+        Return:
+            None
+
+        Raises:
+            ValueError: if value is not a KMIPVersion enumeration
+
+        Example:
+            >>> client.kmip_version = enums.KMIPVersion.KMIP_1_1
+            >>>
+        """
+        if isinstance(value, enums.KMIPVersion):
+            self._kmip_version = value
+        else:
+            raise ValueError("KMIP version must be a KMIPVersion enumeration")
 
     def get_supported_conformance_clauses(self):
         """
@@ -212,15 +271,15 @@ class KMIPProxy(KMIP):
                 self.socket.connect((self.host, self.port))
             except Exception as e:
                 self.logger.error("An error occurred while connecting to "
-                                  "appliance " + self.host)
+                                  "appliance %s: %s", self.host, e)
                 self.socket.close()
-                last_error = e
+                last_error = sys.exc_info()
             else:
                 return
 
         self.socket = None
         if last_error:
-            raise last_error
+            six.reraise(*last_error)
 
     def _create_socket(self, sock):
         self.socket = ssl.wrap_socket(
@@ -244,14 +303,13 @@ class KMIPProxy(KMIP):
             try:
                 self.socket.shutdown(socket.SHUT_RDWR)
                 self.socket.close()
-            except OSError:
+            except (OSError, socket.error):
                 # Can be thrown if the socket is not actually connected to
                 # anything. In this case, ignore the error.
                 pass
             self.socket = None
 
     def create(self, object_type, template_attribute, credential=None):
-        object_type = attr.ObjectType(object_type)
         return self._create(object_type=object_type,
                             template_attribute=template_attribute,
                             credential=credential)
@@ -284,6 +342,84 @@ class KMIPProxy(KMIP):
         """
         return self._activate(uuid, credential=credential)
 
+    def rekey(self,
+              uuid=None,
+              offset=None,
+              template_attribute=None,
+              credential=None):
+        """
+        Check object usage according to specific constraints.
+
+        Args:
+            uuid (string): The unique identifier of a managed cryptographic
+                object that should be checked. Optional, defaults to None.
+            offset (int): An integer specifying, in seconds, the difference
+                between the rekeyed objects initialization date and activation
+                date. Optional, defaults to None.
+            template_attribute (TemplateAttribute): A TemplateAttribute struct
+                containing the attributes to set on the newly rekeyed object.
+                Optional, defaults to None.
+            credential (Credential): A Credential struct containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+
+        Returns:
+            dict: The results of the check operation, containing the following
+                key/value pairs:
+
+                Key                        | Value
+                ---------------------------|-----------------------------------
+                'unique_identifier'        | (string) The unique ID of the
+                                           | checked cryptographic object.
+                'template_attribute'       | (TemplateAttribute) A struct
+                                           | containing attribute set by the
+                                           | server. Optional.
+                'result_status'            | (ResultStatus) An enumeration
+                                           | indicating the status of the
+                                           | operation result.
+                'result_reason'            | (ResultReason) An enumeration
+                                           | providing context for the result
+                                           | status.
+                'result_message'           | (string) A message providing
+                                           | additional context for the
+                                           | operation result.
+        """
+        operation = Operation(OperationEnum.REKEY)
+        request_payload = payloads.RekeyRequestPayload(
+            unique_identifier=uuid,
+            offset=offset,
+            template_attribute=template_attribute
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+
+            if payload.template_attribute is not None:
+                result['template_attribute'] = payload.template_attribute
+
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
     def derive_key(self,
                    object_type,
                    unique_identifiers,
@@ -366,6 +502,107 @@ class KMIPProxy(KMIP):
 
         return result
 
+    def check(self,
+              uuid=None,
+              usage_limits_count=None,
+              cryptographic_usage_mask=None,
+              lease_time=None,
+              credential=None):
+        """
+        Check object usage according to specific constraints.
+
+        Args:
+            uuid (string): The unique identifier of a managed cryptographic
+                object that should be checked. Optional, defaults to None.
+            usage_limits_count (int): An integer specifying the number of
+                items that can be secured with the specified cryptographic
+                object. Optional, defaults to None.
+            cryptographic_usage_mask (list): A list of CryptographicUsageMask
+                enumerations specifying the operations possible with the
+                specified cryptographic object. Optional, defaults to None.
+            lease_time (int): The number of seconds that can be leased for the
+                specified cryptographic object. Optional, defaults to None.
+            credential (Credential): A Credential struct containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+
+        Returns:
+            dict: The results of the check operation, containing the following
+                key/value pairs:
+
+                Key                        | Value
+                ---------------------------|-----------------------------------
+                'unique_identifier'        | (string) The unique ID of the
+                                           | checked cryptographic object.
+                'usage_limits_count'       | (int) The value provided as input
+                                           | if the value exceeds server
+                                           | constraints.
+                'cryptographic_usage_mask' | (list) The value provided as input
+                                           | if the value exceeds server
+                                           | constraints.
+                'lease_time'               | (int) The value provided as input
+                                           | if the value exceeds server
+                                           | constraints.
+                'result_status'            | (ResultStatus) An enumeration
+                                           | indicating the status of the
+                                           | operation result.
+                'result_reason'            | (ResultReason) An enumeration
+                                           | providing context for the result
+                                           | status.
+                'result_message'           | (string) A message providing
+                                           | additional context for the
+                                           | operation result.
+        """
+        # TODO (peter-hamilton) Push this into the Check request.
+        mask = 0
+        for m in cryptographic_usage_mask:
+            mask |= m.value
+
+        operation = Operation(OperationEnum.CHECK)
+        request_payload = payloads.CheckRequestPayload(
+            unique_identifier=uuid,
+            usage_limits_count=usage_limits_count,
+            cryptographic_usage_mask=mask,
+            lease_time=lease_time
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+        if payload.usage_limits_count is not None:
+            result['usage_limits_count'] = payload.usage_limits_count
+        if payload.cryptographic_usage_mask is not None:
+            # TODO (peter-hamilton) Push this into the Check response.
+            masks = []
+            for enumeration in enums.CryptographicUsageMask:
+                if payload.cryptographic_usage_mask & enumeration.value:
+                    masks.append(enumeration)
+            result['cryptographic_usage_mask'] = masks
+        if payload.lease_time is not None:
+            result['lease_time'] = payload.lease_time
+
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
     def get(self, uuid=None, key_format_type=None, key_compression_type=None,
             key_wrapping_specification=None, credential=None):
         return self._get(
@@ -435,7 +672,6 @@ class KMIPProxy(KMIP):
 
     def register(self, object_type, template_attribute, secret,
                  credential=None):
-        object_type = attr.ObjectType(object_type)
         return self._register(object_type=object_type,
                               template_attribute=template_attribute,
                               secret=secret,
@@ -823,7 +1059,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -986,8 +1222,8 @@ class KMIPProxy(KMIP):
         payload_public_key_template_attribute = None
 
         if payload is not None:
-            payload_private_key_uuid = payload.private_key_uuid
-            payload_public_key_uuid = payload.public_key_uuid
+            payload_private_key_uuid = payload.private_key_unique_identifier
+            payload_public_key_uuid = payload.public_key_unique_identifier
             payload_private_key_template_attribute = \
                 payload.private_key_template_attribute
             payload_public_key_template_attribute = \
@@ -1075,7 +1311,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1112,7 +1348,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1145,7 +1381,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1183,7 +1419,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1212,7 +1448,7 @@ class KMIPProxy(KMIP):
         req_pl = payloads.RegisterRequestPayload(
             object_type=object_type,
             template_attribute=template_attribute,
-            secret=secret)
+            managed_object=secret)
         batch_item = messages.RequestBatchItem(operation=operation,
                                                request_payload=req_pl)
 
@@ -1220,7 +1456,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1240,37 +1476,28 @@ class KMIPProxy(KMIP):
         return result
 
     def _locate(self, maximum_items=None, storage_status_mask=None,
-                object_group_member=None, attributes=[], credential=None):
+                object_group_member=None, attributes=None, credential=None):
 
         operation = Operation(OperationEnum.LOCATE)
 
-        mxi = None
-        ssmask = None
-        objgrp = None
+        payload = payloads.LocateRequestPayload(
+            maximum_items=maximum_items,
+            storage_status_mask=storage_status_mask,
+            object_group_member=object_group_member,
+            attributes=attributes
+        )
 
-        if maximum_items is not None:
-            mxi = payloads.LocateRequestPayload.MaximumItems(maximum_items)
-        if storage_status_mask is not None:
-            m = storage_status_mask
-            ssmask = payloads.LocateRequestPayload.StorageStatusMask(m)
-        if object_group_member is not None:
-            o = object_group_member
-            objgrp = payloads.LocateRequestPayload.ObjectGroupMember(o)
-
-        payload = payloads.LocateRequestPayload(maximum_items=mxi,
-                                                storage_status_mask=ssmask,
-                                                object_group_member=objgrp,
-                                                attributes=attributes)
-
-        batch_item = messages.RequestBatchItem(operation=operation,
-                                               request_payload=payload)
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=payload
+        )
 
         message = self._build_request_message(credential, [batch_item])
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
 
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1304,7 +1531,7 @@ class KMIPProxy(KMIP):
         self._send_message(message)
         message = messages.ResponseMessage()
         data = self._receive_message()
-        message.read(data)
+        message.read(data, self.kmip_version)
         batch_items = message.batch_items
         batch_item = batch_items[0]
         payload = batch_item.response_payload
@@ -1340,15 +1567,29 @@ class KMIPProxy(KMIP):
             credential_value)
         return credential
 
+    def _build_protocol_version(self):
+        if self.kmip_version == enums.KMIPVersion.KMIP_1_0:
+            return ProtocolVersion(1, 0)
+        elif self.kmip_version == enums.KMIPVersion.KMIP_1_1:
+            return ProtocolVersion(1, 1)
+        elif self.kmip_version == enums.KMIPVersion.KMIP_1_2:
+            return ProtocolVersion(1, 2)
+        elif self.kmip_version == enums.KMIPVersion.KMIP_1_3:
+            return ProtocolVersion(1, 3)
+        elif self.kmip_version == enums.KMIPVersion.KMIP_1_4:
+            return ProtocolVersion(1, 4)
+        else:
+            return ProtocolVersion(2, 0)
+
     def _build_request_message(self, credential, batch_items):
-        protocol_version = ProtocolVersion.create(1, 2)
+        protocol_version = self._build_protocol_version()
 
         if credential is None:
             credential = self._build_credential()
 
         authentication = None
         if credential is not None:
-            authentication = Authentication(credential)
+            authentication = Authentication([credential])
 
         batch_count = BatchCount(len(batch_items))
         req_header = messages.RequestHeader(protocol_version=protocol_version,
@@ -1360,7 +1601,7 @@ class KMIPProxy(KMIP):
 
     def _send_message(self, message):
         stream = BytearrayStream()
-        message.write(stream)
+        message.write(stream, self.kmip_version)
         self.protocol.write(stream.buffer)
 
     def _receive_message(self):
@@ -1370,14 +1611,14 @@ class KMIPProxy(KMIP):
         self._send_message(request)
         response = messages.ResponseMessage()
         data = self._receive_message()
-        response.read(data)
+        response.read(data, self.kmip_version)
         return response
 
     def _set_variables(self, host, port, keyfile, certfile,
                        cert_reqs, ssl_version, ca_certs,
                        do_handshake_on_connect, suppress_ragged_eofs,
-                       username, password, timeout):
-        conf = ConfigHelper()
+                       username, password, timeout, config_file):
+        conf = ConfigHelper(config_file)
 
         # TODO: set this to a host list
         self.host_list_str = conf.get_valid_value(
