@@ -37,6 +37,7 @@ class KmipServerConfig(object):
         self.settings['enable_tls_client_auth'] = True
         self.settings['tls_cipher_suites'] = []
         self.settings['logging_level'] = logging.INFO
+        self.settings['auth_plugins'] = []
 
         self._expected_settings = [
             'hostname',
@@ -50,7 +51,8 @@ class KmipServerConfig(object):
             'policy_path',
             'enable_tls_client_auth',
             'tls_cipher_suites',
-            'logging_level'
+            'logging_level',
+            'database_path'
         ]
 
     def set_setting(self, setting, value):
@@ -92,8 +94,10 @@ class KmipServerConfig(object):
             self._set_enable_tls_client_auth(value)
         elif setting == 'tls_cipher_suites':
             self._set_tls_cipher_suites(value)
-        else:
+        elif setting == 'logging_level':
             self._set_logging_level(value)
+        else:
+            self._set_database_path(value)
 
     def load_settings(self, path):
         """
@@ -118,9 +122,15 @@ class KmipServerConfig(object):
             "Loading server configuration settings from: {0}".format(path)
         )
 
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
         parser.read(path)
         self._parse_settings(parser)
+        self.parse_auth_settings(parser)
+
+    def parse_auth_settings(self, parser):
+        sections = [x for x in parser.sections() if x.startswith("auth:")]
+        configs = [(x, dict(parser.items(x))) for x in sections]
+        self.settings['auth_plugins'] = configs
 
     def _parse_settings(self, parser):
         if not parser.has_section('server'):
@@ -172,6 +182,8 @@ class KmipServerConfig(object):
             self._set_logging_level(
                 parser.get('server', 'logging_level')
             )
+        if parser.has_option('server', 'database_path'):
+            self._set_database_path(parser.get('server', 'database_path'))
 
     def _set_hostname(self, value):
         if isinstance(value, six.string_types):
@@ -327,3 +339,14 @@ class KmipServerConfig(object):
                 "The logging level must be a string representing a valid "
                 "logging level."
             )
+
+    def _set_database_path(self, value):
+        if not value:
+            self.settings['database_path'] = None
+        elif isinstance(value, six.string_types):
+            self.settings['database_path'] = value
+        else:
+            raise exceptions.ConfigurationError(
+                "The database path, if specified, must be a valid path to a "
+                "SQLite database file."
+            )
