@@ -15,7 +15,6 @@
 
 import copy
 import logging
-import os
 import six
 import sqlalchemy
 
@@ -40,10 +39,6 @@ from kmip.core.messages import messages
 
 from kmip.core.messages import payloads
 
-from kmip.core import misc
-
-from kmip.core import policy as operation_policy
-
 from kmip.pie import factory
 from kmip.pie import objects
 from kmip.pie import sqltypes
@@ -77,7 +72,7 @@ class KmipEngine(object):
         * Cryptographic usage mask enforcement per object type
     """
 
-    def __init__(self, policy_path=None):
+    def __init__(self, policies=None, database_path=None):
         """
         Create a KmipEngine.
 
@@ -85,13 +80,20 @@ class KmipEngine(object):
             policy_path (string): The path to the filesystem directory
                 containing PyKMIP server operation policy JSON files.
                 Optional, defaults to None.
+            database_path (string): The path to the SQLite database file
+                used to store all server data. Optional, defaults to None.
+                If none, database path defaults to '/tmp/pykmip.database'.
         """
         self._logger = logging.getLogger('kmip.server.engine')
 
         self._cryptography_engine = engine.CryptographyEngine()
 
+        self.database_path = 'sqlite:///{}'.format(database_path)
+        if not database_path:
+            self.database_path = 'sqlite:////tmp/pykmip.database'
+
         self._data_store = sqlalchemy.create_engine(
-            'sqlite:////tmp/pykmip.database',
+            self.database_path,
             echo=False,
             connect_args={'check_same_thread': False}
         )
@@ -105,12 +107,16 @@ class KmipEngine(object):
         self._id_placeholder = None
 
         self._protocol_versions = [
-            contents.ProtocolVersion.create(1, 2),
-            contents.ProtocolVersion.create(1, 1),
-            contents.ProtocolVersion.create(1, 0)
+            contents.ProtocolVersion(2, 0),
+            contents.ProtocolVersion(1, 4),
+            contents.ProtocolVersion(1, 3),
+            contents.ProtocolVersion(1, 2),
+            contents.ProtocolVersion(1, 1),
+            contents.ProtocolVersion(1, 0)
         ]
 
-        self._protocol_version = self._protocol_versions[0]
+        self.default_protocol_version = self._protocol_versions[3]
+        self._protocol_version = self._protocol_versions[3]
 
         self._object_map = {
             enums.ObjectType.CERTIFICATE: objects.X509Certificate,
@@ -124,68 +130,8 @@ class KmipEngine(object):
         }
 
         self._attribute_policy = policy.AttributePolicy(self._protocol_version)
-        self._operation_policies = copy.deepcopy(operation_policy.policies)
-        self._load_operation_policies(policy_path)
-
-        self._client_identity = None
-
-    def _load_operation_policies(self, policy_path):
-        if (policy_path is None) or (not os.path.isdir(policy_path)):
-            self._logger.warning(
-                "The specified operation policy directory{0} is not "
-                "valid. No user-defined policies will be loaded.".format(
-                    " (" + policy_path + ")" if policy_path else ''
-                )
-            )
-            return dict()
-        else:
-            self._logger.info(
-                "Loading user-defined operation policy files from: {0}".format(
-                    policy_path
-                )
-            )
-
-        for filename in os.listdir(policy_path):
-            file_path = os.path.join(policy_path, filename)
-            if os.path.isfile(file_path):
-                self._logger.info(
-                    "Loading user-defined operation policies "
-                    "from file: {0}".format(file_path)
-                )
-
-                try:
-                    policies = operation_policy.read_policy_from_file(
-                        file_path
-                    )
-                except ValueError as e:
-                    self._logger.error(
-                        "A failure occurred while loading policies."
-                    )
-                    self._logger.exception(e)
-                    continue
-
-                reserved_policies = ['default', 'public']
-                for policy_name in six.iterkeys(policies):
-                    if policy_name in reserved_policies:
-                        self._logger.warning(
-                            "Loaded policy '{0}' overwrites a reserved "
-                            "policy and will be thrown out.".format(
-                                policy_name
-                            )
-                        )
-                    elif policy_name in six.iterkeys(
-                            self._operation_policies
-                    ):
-                        self._logger.warning(
-                            "Loaded policy '{0}' overwrites a "
-                            "preexisting policy and will be thrown "
-                            "out.".format(policy_name)
-                        )
-                    else:
-                        self._operation_policies.update([(
-                            policy_name,
-                            policies.get(policy_name)
-                        )])
+        self._operation_policies = policies
+        self._client_identity = [None, None]
 
     def _get_enum_string(self, e):
         return ''.join([x.capitalize() for x in e.name.split('_')])
@@ -262,11 +208,16 @@ class KmipEngine(object):
             ResponseMessage: The response containing all of the results from
                 the request batch items.
         """
-        self._client_identity = None
+        self._client_identity = [None, None]
         header = request.request_header
 
         # Process the protocol version
         self._set_protocol_version(header.protocol_version)
+        self._logger.debug(
+            "Request specified KMIP version: {0}".format(
+                header.protocol_version
+            )
+        )
 
         # Process the maximum response size
         max_response_size = None
@@ -328,9 +279,13 @@ class KmipEngine(object):
 
         # Process the authentication credentials
         if header.authentication:
-            auth_credentials = header.authentication.credential
+            if header.authentication.credentials:
+                auth_credentials = header.authentication.credentials[0]
+            else:
+                auth_credentials = None
         else:
             auth_credentials = None
+
         self._verify_credential(auth_credentials, credential)
 
         # Process the batch error continuation option
@@ -358,7 +313,7 @@ class KmipEngine(object):
             response_batch
         )
 
-        return response, max_response_size
+        return response, max_response_size, header.protocol_version
 
     def _build_response(self, version, batch_items):
         header = messages.ResponseHeader(
@@ -638,10 +593,14 @@ class KmipEngine(object):
         for attribute_name in attr_names:
             object_type = managed_object._object_type
 
+            # TODO (ph) Create the policy once and just pass these calls the
+            #           KMIP version for the current request.
             if not self._attribute_policy.is_attribute_supported(
                     attribute_name
             ):
                 continue
+            if self._attribute_policy.is_attribute_deprecated(attribute_name):
+                continue
 
             if self._attribute_policy.is_attribute_applicable_to_object_type(
                 attribute_name,
@@ -850,24 +809,94 @@ class KmipEngine(object):
 
     def _is_allowed_by_operation_policy(
             self,
-            operation_policy,
+            policy_name,
             session_identity,
             object_owner,
             object_type,
             operation
     ):
-        policy_set = self._operation_policies.get(operation_policy)
-        if not policy_set:
-            self._logger.warning(
-                "The '{0}' policy does not exist.".format(operation_policy)
+        session_user = session_identity[0]
+        session_groups = session_identity[1]
+
+        if session_groups is None:
+            session_groups = [None]
+
+        for session_group in session_groups:
+            allowed = self.is_allowed(
+                policy_name,
+                session_user,
+                session_group,
+                object_owner,
+                object_type,
+                operation
             )
+            if allowed:
+                return True
+
+        return False
+
+    def get_relevant_policy_section(self, policy_name, group=None):
+        """
+        Look up the policy corresponding to the provided policy name and
+        group (optional). Log any issues found during the look up.
+        """
+        policy_bundle = self._operation_policies.get(policy_name)
+
+        if not policy_bundle:
+            self._logger.warning(
+                "The '{}' policy does not exist.".format(policy_name)
+            )
+            return None
+
+        if group:
+            groups_policy_bundle = policy_bundle.get('groups')
+            if not groups_policy_bundle:
+                self._logger.debug(
+                    "The '{}' policy does not support groups.".format(
+                        policy_name
+                    )
+                )
+                return None
+            else:
+                group_policy = groups_policy_bundle.get(group)
+                if not group_policy:
+                    self._logger.debug(
+                        "The '{}' policy does not support group '{}'.".format(
+                            policy_name,
+                            group
+                        )
+                    )
+                    return None
+                else:
+                    return group_policy
+        else:
+            return policy_bundle.get('preset')
+
+    def is_allowed(
+            self,
+            policy_name,
+            session_user,
+            session_group,
+            object_owner,
+            object_type,
+            operation
+    ):
+        """
+        Determine if object access is allowed for the provided policy and
+        session settings.
+        """
+        policy_section = self.get_relevant_policy_section(
+            policy_name,
+            session_group
+        )
+        if policy_section is None:
             return False
 
-        object_policy = policy_set.get(object_type)
+        object_policy = policy_section.get(object_type)
         if not object_policy:
             self._logger.warning(
                 "The '{0}' policy does not apply to {1} objects.".format(
-                    operation_policy,
+                    policy_name,
                     self._get_enum_string(object_type)
                 )
             )
@@ -878,7 +907,7 @@ class KmipEngine(object):
             self._logger.warning(
                 "The '{0}' policy does not apply to {1} operations on {2} "
                 "objects.".format(
-                    operation_policy,
+                    policy_name,
                     self._get_enum_string(operation),
                     self._get_enum_string(object_type)
                 )
@@ -888,7 +917,7 @@ class KmipEngine(object):
         if operation_object_policy == enums.Policy.ALLOW_ALL:
             return True
         elif operation_object_policy == enums.Policy.ALLOW_OWNER:
-            if session_identity == object_owner:
+            if session_user == object_owner:
                 return True
             else:
                 return False
@@ -908,6 +937,8 @@ class KmipEngine(object):
             object_type.unique_identifier == uid
         ).one()
 
+        # TODO (peter-hamilton) Add debug log with policy contents?
+
         # Determine if the request should be carried out under the object's
         # operation policy. If not, feign ignorance of the object.
         is_allowed = self._is_allowed_by_operation_policy(
@@ -918,7 +949,7 @@ class KmipEngine(object):
             operation
         )
         if not is_allowed:
-            raise exceptions.ItemNotFound(
+            raise exceptions.PermissionDenied(
                 "Could not locate object: {0}".format(uid)
             )
 
@@ -994,7 +1025,7 @@ class KmipEngine(object):
     def _process_create(self, payload):
         self._logger.info("Processing operation: Create")
 
-        object_type = payload.object_type.value
+        object_type = payload.object_type
         template_attribute = payload.template_attribute
 
         if object_type != enums.ObjectType.SYMMETRIC_KEY:
@@ -1057,7 +1088,7 @@ class KmipEngine(object):
         )
 
         # TODO (peterhamilton) Set additional server-only attributes.
-        managed_object._owner = self._client_identity
+        managed_object._owner = self._client_identity[0]
         managed_object.initial_date = int(time.time())
 
         self._data_session.add(managed_object)
@@ -1074,9 +1105,7 @@ class KmipEngine(object):
 
         response_payload = payloads.CreateResponsePayload(
             object_type=payload.object_type,
-            unique_identifier=attributes.UniqueIdentifier(
-                str(managed_object.unique_identifier)
-            ),
+            unique_identifier=str(managed_object.unique_identifier),
             template_attribute=None
         )
 
@@ -1225,9 +1254,9 @@ class KmipEngine(object):
         )
 
         # TODO (peterhamilton) Set additional server-only attributes.
-        public_key._owner = self._client_identity
+        public_key._owner = self._client_identity[0]
         public_key.initial_date = int(time.time())
-        private_key._owner = self._client_identity
+        private_key._owner = self._client_identity[0]
         private_key.initial_date = public_key.initial_date
 
         self._data_session.add(public_key)
@@ -1249,12 +1278,8 @@ class KmipEngine(object):
         )
 
         response_payload = payloads.CreateKeyPairResponsePayload(
-            private_key_uuid=attributes.PrivateKeyUniqueIdentifier(
-                str(private_key.unique_identifier)
-            ),
-            public_key_uuid=attributes.PublicKeyUniqueIdentifier(
-                str(public_key.unique_identifier)
-            )
+            private_key_unique_identifier=str(private_key.unique_identifier),
+            public_key_unique_identifier=str(public_key.unique_identifier)
         )
 
         self._id_placeholder = str(private_key.unique_identifier)
@@ -1264,7 +1289,7 @@ class KmipEngine(object):
     def _process_register(self, payload):
         self._logger.info("Processing operation: Register")
 
-        object_type = payload.object_type.value
+        object_type = payload.object_type
         template_attribute = payload.template_attribute
 
         if self._object_map.get(object_type) is None:
@@ -1277,8 +1302,8 @@ class KmipEngine(object):
                 )
             )
 
-        if payload.secret:
-            secret = payload.secret
+        if payload.managed_object:
+            secret = payload.managed_object
         else:
             # TODO (peterhamilton) It is possible to register 'empty' secrets
             # like Private Keys. For now, that feature is not supported.
@@ -1302,7 +1327,7 @@ class KmipEngine(object):
         )
 
         # TODO (peterhamilton) Set additional server-only attributes.
-        managed_object._owner = self._client_identity
+        managed_object._owner = self._client_identity[0]
         managed_object.initial_date = int(time.time())
 
         self._data_session.add(managed_object)
@@ -1319,9 +1344,7 @@ class KmipEngine(object):
         )
 
         response_payload = payloads.RegisterResponsePayload(
-            unique_identifier=attributes.UniqueIdentifier(
-                str(managed_object.unique_identifier)
-            )
+            unique_identifier=str(managed_object.unique_identifier)
         )
 
         self._id_placeholder = str(managed_object.unique_identifier)
@@ -1424,7 +1447,7 @@ class KmipEngine(object):
             else:
                 raise exceptions.InvalidField(
                     "The cryptographic length must correspond to a valid "
-                    "number of bytes (i.e., it must be a multiple of 8)."
+                    "number of bytes; it must be a multiple of 8."
                 )
         else:
             raise exceptions.InvalidField(
@@ -1490,7 +1513,7 @@ class KmipEngine(object):
         )
 
         # TODO (peterhamilton) Set additional server-only attributes.
-        managed_object._owner = self._client_identity
+        managed_object._owner = self._client_identity[0]
         managed_object.initial_date = int(time.time())
 
         self._data_session.add(managed_object)
@@ -1537,16 +1560,16 @@ class KmipEngine(object):
                     if attribute_name == 'Name':
                         names = attr
                         if attribute_value not in names:
-                                break
+                            break
                     # TODO: filtering on other attributes
                 else:
                     managed_objects_filtered.append(managed_object)
 
             managed_objects = managed_objects_filtered
 
-        unique_identifiers = [attributes.UniqueIdentifier(
-                            str(managed_object.unique_identifier))
-                            for managed_object in managed_objects]
+        unique_identifiers = [
+            str(x.unique_identifier) for x in managed_objects
+        ]
 
         response_payload = payloads.LocateResponsePayload(
             unique_identifiers=unique_identifiers
@@ -1902,7 +1925,7 @@ class KmipEngine(object):
     def _process_query(self, payload):
         self._logger.info("Processing operation: Query")
 
-        queries = [x.value for x in payload.query_functions]
+        queries = payload.query_functions
 
         operations = list()
         objects = list()
@@ -1913,38 +1936,38 @@ class KmipEngine(object):
 
         if enums.QueryFunction.QUERY_OPERATIONS in queries:
             operations = list([
-                contents.Operation(enums.Operation.CREATE),
-                contents.Operation(enums.Operation.CREATE_KEY_PAIR),
-                contents.Operation(enums.Operation.REGISTER),
-                contents.Operation(enums.Operation.DERIVE_KEY),
-                contents.Operation(enums.Operation.LOCATE),
-                contents.Operation(enums.Operation.GET),
-                contents.Operation(enums.Operation.GET_ATTRIBUTES),
-                contents.Operation(enums.Operation.GET_ATTRIBUTE_LIST),
-                contents.Operation(enums.Operation.ACTIVATE),
-                contents.Operation(enums.Operation.REVOKE),
-                contents.Operation(enums.Operation.DESTROY),
-                contents.Operation(enums.Operation.QUERY)
+                enums.Operation.CREATE,
+                enums.Operation.CREATE_KEY_PAIR,
+                enums.Operation.REGISTER,
+                enums.Operation.DERIVE_KEY,
+                enums.Operation.LOCATE,
+                enums.Operation.GET,
+                enums.Operation.GET_ATTRIBUTES,
+                enums.Operation.GET_ATTRIBUTE_LIST,
+                enums.Operation.ACTIVATE,
+                enums.Operation.REVOKE,
+                enums.Operation.DESTROY,
+                enums.Operation.QUERY
             ])
 
-            if self._protocol_version >= contents.ProtocolVersion.create(1, 1):
+            if self._protocol_version >= contents.ProtocolVersion(1, 1):
                 operations.extend([
-                    contents.Operation(enums.Operation.DISCOVER_VERSIONS)
+                    enums.Operation.DISCOVER_VERSIONS
                 ])
-            if self._protocol_version >= contents.ProtocolVersion.create(1, 2):
+            if self._protocol_version >= contents.ProtocolVersion(1, 2):
                 operations.extend([
-                    contents.Operation(enums.Operation.ENCRYPT),
-                    contents.Operation(enums.Operation.DECRYPT),
-                    contents.Operation(enums.Operation.SIGN),
-                    contents.Operation(enums.Operation.SIGNATURE_VERIFY),
-                    contents.Operation(enums.Operation.MAC)
+                    enums.Operation.ENCRYPT,
+                    enums.Operation.DECRYPT,
+                    enums.Operation.SIGN,
+                    enums.Operation.SIGNATURE_VERIFY,
+                    enums.Operation.MAC
                 ])
 
         if enums.QueryFunction.QUERY_OBJECTS in queries:
             objects = list()
         if enums.QueryFunction.QUERY_SERVER_INFORMATION in queries:
-            vendor_identification = misc.VendorIdentification(
-                "PyKMIP {0} Software Server".format(kmip.__version__)
+            vendor_identification = "PyKMIP {0} Software Server".format(
+                kmip.__version__
             )
             server_information = None
         if enums.QueryFunction.QUERY_APPLICATION_NAMESPACES in queries:
@@ -2023,12 +2046,12 @@ class KmipEngine(object):
                 "for encryption."
             )
 
-        if enums.CryptographicUsageMask.ENCRYPT not in \
-           managed_object.cryptographic_usage_masks:
-                raise exceptions.PermissionDenied(
-                    "The Encrypt bit must be set in the encryption key's "
-                    "cryptographic usage mask."
-                )
+        masks = managed_object.cryptographic_usage_masks
+        if enums.CryptographicUsageMask.ENCRYPT not in masks:
+            raise exceptions.PermissionDenied(
+                "The Encrypt bit must be set in the encryption key's "
+                "cryptographic usage mask."
+            )
 
         result = self._cryptography_engine.encrypt(
             cryptographic_parameters.cryptographic_algorithm,
@@ -2086,12 +2109,12 @@ class KmipEngine(object):
                 "for decryption."
             )
 
-        if enums.CryptographicUsageMask.DECRYPT not in \
-           managed_object.cryptographic_usage_masks:
-                raise exceptions.PermissionDenied(
-                    "The Decrypt bit must be set in the decryption key's "
-                    "cryptographic usage mask."
-                )
+        masks = managed_object.cryptographic_usage_masks
+        if enums.CryptographicUsageMask.DECRYPT not in masks:
+            raise exceptions.PermissionDenied(
+                "The Decrypt bit must be set in the decryption key's "
+                "cryptographic usage mask."
+            )
 
         result = self._cryptography_engine.decrypt(
             cryptographic_parameters.cryptographic_algorithm,
@@ -2229,12 +2252,12 @@ class KmipEngine(object):
                 "Object is not in a state that can be used for MACing."
             )
 
-        if enums.CryptographicUsageMask.MAC_GENERATE not in \
-           managed_object.cryptographic_usage_masks:
-                raise exceptions.PermissionDenied(
-                    "MAC Generate must be set in the object's cryptographic "
-                    "usage mask"
-                )
+        masks = managed_object.cryptographic_usage_masks
+        if enums.CryptographicUsageMask.MAC_GENERATE not in masks:
+            raise exceptions.PermissionDenied(
+                "MAC Generate must be set in the object's cryptographic "
+                "usage mask"
+            )
 
         result = self._cryptography_engine.mac(
             algorithm,
