@@ -20,6 +20,7 @@ from kmip.core.attributes import CryptographicAlgorithm
 from kmip.core.attributes import CryptographicLength
 from kmip.core.attributes import Name
 
+from kmip.core import enums
 from kmip.core.enums import AttributeType
 from kmip.core.enums import CryptographicAlgorithm as CryptoAlgorithmEnum
 from kmip.core.enums import CryptographicUsageMask
@@ -31,7 +32,7 @@ from kmip.core.enums import OpaqueDataType
 from kmip.core.enums import SecretDataType
 from kmip.core.enums import ResultStatus
 from kmip.core.enums import ResultReason
-from kmip.core.enums import QueryFunction as QueryFunctionEnum
+from kmip.core.enums import QueryFunction
 
 from kmip.core.factories.attributes import AttributeFactory
 from kmip.core.factories.credentials import CredentialFactory
@@ -44,11 +45,6 @@ from kmip.core.objects import KeyBlock
 from kmip.core.objects import KeyMaterial
 from kmip.core.objects import KeyValue
 from kmip.core.objects import TemplateAttribute
-from kmip.core.objects import PrivateKeyTemplateAttribute
-from kmip.core.objects import PublicKeyTemplateAttribute
-from kmip.core.objects import CommonTemplateAttribute
-
-from kmip.core.misc import QueryFunction
 
 from kmip.core.secrets import SymmetricKey
 from kmip.core.secrets import PrivateKey
@@ -149,11 +145,18 @@ class TestIntegration(TestCase):
         private_key_attributes = [priv_name]
         public_key_attributes = [pub_name]
 
-        common = CommonTemplateAttribute(attributes=common_attributes)
-        priv_templ_attr = PrivateKeyTemplateAttribute(
-            attributes=private_key_attributes)
-        pub_templ_attr = PublicKeyTemplateAttribute(
-            attributes=public_key_attributes)
+        common = TemplateAttribute(
+            attributes=common_attributes,
+            tag=enums.Tags.COMMON_TEMPLATE_ATTRIBUTE
+        )
+        priv_templ_attr = TemplateAttribute(
+            attributes=private_key_attributes,
+            tag=enums.Tags.PRIVATE_KEY_TEMPLATE_ATTRIBUTE
+        )
+        pub_templ_attr = TemplateAttribute(
+            attributes=public_key_attributes,
+            tag=enums.Tags.PUBLIC_KEY_TEMPLATE_ATTRIBUTE
+        )
 
         return self.client.\
             create_key_pair(common_template_attribute=common,
@@ -309,19 +312,16 @@ class TestIntegration(TestCase):
 
     def test_query(self):
         # Build query function list, asking for all server data.
-        query_functions = list()
-        query_functions.append(
-            QueryFunction(QueryFunctionEnum.QUERY_OPERATIONS))
-        query_functions.append(
-            QueryFunction(QueryFunctionEnum.QUERY_OBJECTS))
-        query_functions.append(
-            QueryFunction(QueryFunctionEnum.QUERY_SERVER_INFORMATION))
-        query_functions.append(
-            QueryFunction(QueryFunctionEnum.QUERY_APPLICATION_NAMESPACES))
-        query_functions.append(
-            QueryFunction(QueryFunctionEnum.QUERY_EXTENSION_LIST))
-        query_functions.append(
-            QueryFunction(QueryFunctionEnum.QUERY_EXTENSION_MAP))
+        query_functions = list(
+            [
+                QueryFunction.QUERY_OPERATIONS,
+                QueryFunction.QUERY_OBJECTS,
+                QueryFunction.QUERY_SERVER_INFORMATION,
+                QueryFunction.QUERY_APPLICATION_NAMESPACES,
+                QueryFunction.QUERY_EXTENSION_LIST,
+                QueryFunction.QUERY_EXTENSION_MAP
+            ]
+        )
 
         result = self.client.query(query_functions=query_functions)
 
@@ -338,11 +338,11 @@ class TestIntegration(TestCase):
         result = self._create_symmetric_key(key_name=key_name)
 
         self._check_result_status(result, ResultStatus, ResultStatus.SUCCESS)
-        self._check_object_type(result.object_type.value, ObjectType,
+        self._check_object_type(result.object_type, ObjectType,
                                 ObjectType.SYMMETRIC_KEY)
-        self._check_uuid(result.uuid.value, str)
+        self._check_uuid(result.uuid, str)
 
-        result = self.client.get(uuid=result.uuid.value, credential=None)
+        result = self.client.get(uuid=result.uuid, credential=None)
 
         self._check_result_status(result, ResultStatus, ResultStatus.SUCCESS)
         self._check_object_type(result.object_type, ObjectType,
@@ -427,10 +427,10 @@ class TestIntegration(TestCase):
                                       credential=None)
 
         self._check_result_status(result, ResultStatus, ResultStatus.SUCCESS)
-        self._check_uuid(result.uuid.value, str)
+        self._check_uuid(result.uuid, str)
 
         # Check that the returned key bytes match what was provided
-        uuid = result.uuid.value
+        uuid = result.uuid
         result = self.client.get(uuid=uuid, credential=None)
 
         self._check_result_status(result, ResultStatus, ResultStatus.SUCCESS)
@@ -488,12 +488,12 @@ class TestIntegration(TestCase):
         self._check_result_status(result, ResultStatus, ResultStatus.SUCCESS)
 
         # Check UUID value for Private key
-        self._check_uuid(result.private_key_uuid.value, str)
+        self._check_uuid(result.private_key_uuid, str)
         # Check UUID value for Public key
-        self._check_uuid(result.public_key_uuid.value, str)
+        self._check_uuid(result.public_key_uuid, str)
 
-        priv_key_uuid = result.private_key_uuid.value
-        pub_key_uuid = result.public_key_uuid.value
+        priv_key_uuid = result.private_key_uuid
+        pub_key_uuid = result.public_key_uuid
 
         priv_key_result = self.client.get(uuid=priv_key_uuid, credential=None)
         pub_key_result = self.client.get(uuid=pub_key_uuid, credential=None)
@@ -522,17 +522,17 @@ class TestIntegration(TestCase):
         self.assertIsInstance(pub_secret, pub_expected)
 
         self.logger.debug('Destroying key: ' + key_name + ' Private' +
-                          '\n With UUID: ' + result.private_key_uuid.value)
+                          '\n With UUID: ' + result.private_key_uuid)
         destroy_priv_key_result = self.client.destroy(
-            result.private_key_uuid.value)
+            result.private_key_uuid)
 
         self._check_result_status(destroy_priv_key_result, ResultStatus,
                                   ResultStatus.SUCCESS)
 
         self.logger.debug('Destroying key: ' + key_name + ' Public' +
-                          '\n With UUID: ' + result.public_key_uuid.value)
+                          '\n With UUID: ' + result.public_key_uuid)
         destroy_pub_key_result = self.client.destroy(
-            result.public_key_uuid.value)
+            result.public_key_uuid)
         self._check_result_status(destroy_pub_key_result, ResultStatus,
                                   ResultStatus.SUCCESS)
 
@@ -654,10 +654,10 @@ class TestIntegration(TestCase):
         self._check_result_status(priv_key_result, ResultStatus,
                                   ResultStatus.SUCCESS)
 
-        self._check_uuid(priv_key_result.uuid.value, str)
+        self._check_uuid(priv_key_result.uuid, str)
 
         # Check that the returned key bytes match what was provided
-        priv_uuid = priv_key_result.uuid.value
+        priv_uuid = priv_key_result.uuid
 
         priv_key_result = self.client.get(uuid=priv_uuid, credential=None)
 
@@ -766,7 +766,7 @@ class TestIntegration(TestCase):
         self._check_result_status(pub_key_result, ResultStatus,
                                   ResultStatus.SUCCESS)
         # Check that the returned key bytes match what was provided
-        pub_uuid = pub_key_result.uuid.value
+        pub_uuid = pub_key_result.uuid
         pub_key_result = self.client.get(uuid=pub_uuid, credential=None)
         self._check_result_status(pub_key_result, ResultStatus,
                                   ResultStatus.SUCCESS)
@@ -901,10 +901,10 @@ class TestIntegration(TestCase):
         self._check_result_status(cert_result, ResultStatus,
                                   ResultStatus.SUCCESS)
 
-        self._check_uuid(cert_result.uuid.value, str)
+        self._check_uuid(cert_result.uuid, str)
 
         # Check that the returned key bytes match what was provided
-        cert_uuid = cert_result.uuid.value
+        cert_uuid = cert_result.uuid
 
         cert_result = self.client.get(uuid=cert_uuid, credential=None)
 
@@ -1008,10 +1008,10 @@ class TestIntegration(TestCase):
         self._check_result_status(pass_result, ResultStatus,
                                   ResultStatus.SUCCESS)
 
-        self._check_uuid(pass_result.uuid.value, str)
+        self._check_uuid(pass_result.uuid, str)
 
         # Check that the returned key bytes match what was provided
-        pass_uuid = pass_result.uuid.value
+        pass_uuid = pass_result.uuid
 
         pass_result = self.client.get(uuid=pass_uuid, credential=None)
 
@@ -1102,10 +1102,10 @@ class TestIntegration(TestCase):
         self._check_result_status(opaque_obj_result, ResultStatus,
                                   ResultStatus.SUCCESS)
 
-        self._check_uuid(opaque_obj_result.uuid.value, str)
+        self._check_uuid(opaque_obj_result.uuid, str)
 
         # Check that the returned key bytes match what was provided
-        opaque_obj_uuid = opaque_obj_result.uuid.value
+        opaque_obj_uuid = opaque_obj_result.uuid
 
         opaque_obj_result = self.client.get(uuid=opaque_obj_uuid,
                                             credential=None)
@@ -1160,10 +1160,10 @@ class TestIntegration(TestCase):
         """
         key_name = 'Integration Test - Create-GetAttributeList-Destroy Key'
         result = self._create_symmetric_key(key_name=key_name)
-        uid = result.uuid.value
+        uid = result.uuid
 
         self.assertEqual(ResultStatus.SUCCESS, result.result_status.value)
-        self.assertEqual(ObjectType.SYMMETRIC_KEY, result.object_type.value)
+        self.assertEqual(ObjectType.SYMMETRIC_KEY, result.object_type)
         self.assertIsInstance(uid, str)
 
         try:
