@@ -111,7 +111,9 @@ class TestProxyKmipClientIntegration(testtools.TestCase):
             enums.CryptographicAlgorithm.AES,
             128,
             (b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E'
-             b'\x0F'))
+             b'\x0F'),
+            name="Test Symmetric Key"
+        )
 
         uid = self.client.register(key)
         self.assertIsInstance(uid, six.string_types)
@@ -179,7 +181,11 @@ class TestProxyKmipClientIntegration(testtools.TestCase):
         asymmetric key pair.
         """
         public_uid, private_uid = self.client.create_key_pair(
-            enums.CryptographicAlgorithm.RSA, 2048)
+            enums.CryptographicAlgorithm.RSA,
+            2048,
+            public_usage_mask=[enums.CryptographicUsageMask.ENCRYPT],
+            private_usage_mask=[enums.CryptographicUsageMask.DECRYPT]
+        )
         self.assertIsInstance(public_uid, six.string_types)
         self.assertIsInstance(private_uid, six.string_types)
 
