@@ -16,6 +16,7 @@
 import logging
 import mock
 
+import six
 from six.moves import configparser
 
 import testtools
@@ -67,12 +68,13 @@ class TestKmipServerConfig(testtools.TestCase):
         c._set_enable_tls_client_auth = mock.MagicMock()
         c._set_tls_cipher_suites = mock.MagicMock()
         c._set_logging_level = mock.MagicMock()
+        c._set_database_path = mock.MagicMock()
 
         # Test the right error is generated when setting an unsupported
         # setting.
         args = ('invalid', None)
         regex = "Setting 'invalid' is not supported."
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c.set_setting,
@@ -114,6 +116,9 @@ class TestKmipServerConfig(testtools.TestCase):
         c.set_setting('logging_level', 'WARNING')
         c._set_logging_level.assert_called_once_with('WARNING')
 
+        c.set_setting('database_path', '/var/pykmip/pykmip.db')
+        c._set_database_path.assert_called_once_with('/var/pykmip/pykmip.db')
+
     def test_load_settings(self):
         """
         Test that the right calls are made and the right errors generated when
@@ -123,13 +128,14 @@ class TestKmipServerConfig(testtools.TestCase):
         c = config.KmipServerConfig()
         c._logger = mock.MagicMock()
         c._parse_settings = mock.MagicMock()
+        c.parse_auth_settings = mock.MagicMock()
 
         # Test that the right calls are made when correctly processing the
         # configuration file.
         with mock.patch('os.path.exists') as os_mock:
             os_mock.return_value = True
             with mock.patch(
-                'six.moves.configparser.SafeConfigParser.read'
+                'six.moves.configparser.ConfigParser.read'
             ) as parser_mock:
                 c.load_settings("/test/path/server.conf")
                 c._logger.info.assert_any_call(
@@ -138,6 +144,7 @@ class TestKmipServerConfig(testtools.TestCase):
                 )
                 parser_mock.assert_called_with("/test/path/server.conf")
                 self.assertTrue(c._parse_settings.called)
+                self.assertTrue(c.parse_auth_settings.called)
 
         # Test that a ConfigurationError is generated when the path is invalid.
         c._logger.reset_mock()
@@ -151,6 +158,66 @@ class TestKmipServerConfig(testtools.TestCase):
                 *args
             )
 
+    def test_parse_auth_settings(self):
+        """
+        Test that server authentication plugin settings are parsed correctly.
+        """
+        parser = configparser.ConfigParser()
+        parser.add_section('server')
+        parser.add_section('auth:slugs')
+        parser.set('auth:slugs', 'enabled', 'True')
+        parser.set('auth:slugs', 'url', 'http://127.0.0.1:8080/slugs/')
+        parser.add_section('auth:ldap')
+        parser.set('auth:ldap', 'enabled', 'False')
+        parser.set('auth:ldap', 'url', 'http://127.0.0.1:8080/ldap/')
+
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        self.assertEqual([], c.settings['auth_plugins'])
+
+        c.parse_auth_settings(parser)
+        configs = c.settings['auth_plugins']
+
+        self.assertIsInstance(configs, list)
+        self.assertEqual(2, len(configs))
+
+        for c in configs:
+            self.assertIsInstance(c, tuple)
+            self.assertEqual(2, len(c))
+            self.assertIn(c[0], ['auth:slugs', 'auth:ldap'])
+            self.assertIsInstance(c[1], dict)
+
+            if c[0] == 'auth:slugs':
+                self.assertIn('enabled', six.iterkeys(c[1]))
+                self.assertEqual('True', c[1]['enabled'])
+                self.assertIn('url', six.iterkeys(c[1]))
+                self.assertEqual('http://127.0.0.1:8080/slugs/', c[1]['url'])
+            elif c[0] == 'auth:ldap':
+                self.assertIn('enabled', six.iterkeys(c[1]))
+                self.assertEqual('False', c[1]['enabled'])
+                self.assertIn('url', six.iterkeys(c[1]))
+                self.assertEqual('http://127.0.0.1:8080/ldap/', c[1]['url'])
+
+    def test_parse_auth_settings_no_config(self):
+        """
+        Test that server authentication plugin settings are parsed correctly,
+        even when not specified.
+        """
+        parser = configparser.ConfigParser()
+        parser.add_section('server')
+
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        self.assertEqual([], c.settings['auth_plugins'])
+
+        c.parse_auth_settings(parser)
+        configs = c.settings['auth_plugins']
+
+        self.assertIsInstance(configs, list)
+        self.assertEqual(0, len(configs))
+
     def test_parse_settings(self):
         """
         Test that the right methods are called and the right errors generated
@@ -169,9 +236,10 @@ class TestKmipServerConfig(testtools.TestCase):
         c._set_enable_tls_client_auth = mock.MagicMock()
         c._set_tls_cipher_suites = mock.MagicMock()
         c._set_logging_level = mock.MagicMock()
+        c._set_database_path = mock.MagicMock()
 
         # Test that the right calls are made when correctly parsing settings.
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
         parser.add_section('server')
         parser.set('server', 'hostname', '127.0.0.1')
         parser.set('server', 'port', '5696')
@@ -187,6 +255,7 @@ class TestKmipServerConfig(testtools.TestCase):
             "\n    TLS_RSA_WITH_AES_256_CBC_SHA256"
         )
         parser.set('server', 'logging_level', 'ERROR')
+        parser.set('server', 'database_path', '/var/pykmip/pykmip.db')
 
         c._parse_settings(parser)
 
@@ -204,16 +273,17 @@ class TestKmipServerConfig(testtools.TestCase):
             "\n    TLS_RSA_WITH_AES_256_CBC_SHA256"
         )
         c._set_logging_level.assert_called_once_with('ERROR')
+        c._set_database_path.assert_called_once_with('/var/pykmip/pykmip.db')
 
         # Test that a ConfigurationError is generated when the expected
         # section is missing.
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
 
         args = (parser, )
         regex = (
             "The server configuration file does not have a 'server' section."
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._parse_settings,
@@ -222,7 +292,7 @@ class TestKmipServerConfig(testtools.TestCase):
 
         # Test that a ConfigurationError is generated when an unexpected
         # setting is provided.
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
         parser.add_section('server')
         parser.set('server', 'invalid', 'invalid')
 
@@ -231,7 +301,7 @@ class TestKmipServerConfig(testtools.TestCase):
             "Setting 'invalid' is not a supported setting. Please remove it "
             "from the configuration file."
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._parse_settings,
@@ -240,14 +310,14 @@ class TestKmipServerConfig(testtools.TestCase):
 
         # Test that a ConfigurationError is generated when an expected
         # setting is missing.
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
         parser.add_section('server')
 
         args = (parser, )
         regex = (
             "Setting 'hostname' is missing from the configuration file."
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._parse_settings,
@@ -256,14 +326,14 @@ class TestKmipServerConfig(testtools.TestCase):
 
         # Test that a ConfigurationError is generated when an expected
         # setting is missing.
-        parser = configparser.SafeConfigParser()
+        parser = configparser.ConfigParser()
         parser.add_section('server')
 
         args = (parser, )
         regex = (
             "Setting 'hostname' is missing from the configuration file."
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._parse_settings,
@@ -286,7 +356,7 @@ class TestKmipServerConfig(testtools.TestCase):
         # value.
         args = (0, )
         regex = "The hostname value must be a string."
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._set_hostname,
@@ -310,7 +380,7 @@ class TestKmipServerConfig(testtools.TestCase):
         # value.
         args = ('invalid', )
         regex = "The port value must be an integer in the range 0 - 65535."
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._set_port,
@@ -320,7 +390,7 @@ class TestKmipServerConfig(testtools.TestCase):
 
         args = (65536, )
         regex = "The port value must be an integer in the range 0 - 65535."
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._set_port,
@@ -362,7 +432,7 @@ class TestKmipServerConfig(testtools.TestCase):
             "The certificate path value, if specified, must be a valid "
             "string path to a certificate file."
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._set_certificate_path,
@@ -377,7 +447,7 @@ class TestKmipServerConfig(testtools.TestCase):
         )
         with mock.patch('os.path.exists') as os_mock:
             os_mock.return_value = False
-            self.assertRaisesRegexp(
+            self.assertRaisesRegex(
                 exceptions.ConfigurationError,
                 regex,
                 c._set_certificate_path,
@@ -421,7 +491,7 @@ class TestKmipServerConfig(testtools.TestCase):
             "The key path value, if specified, must be a valid string path "
             "to a certificate key file."
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._set_key_path,
@@ -436,7 +506,7 @@ class TestKmipServerConfig(testtools.TestCase):
         )
         with mock.patch('os.path.exists') as os_mock:
             os_mock.return_value = False
-            self.assertRaisesRegexp(
+            self.assertRaisesRegex(
                 exceptions.ConfigurationError,
                 regex,
                 c._set_key_path,
@@ -515,7 +585,7 @@ class TestKmipServerConfig(testtools.TestCase):
             "The authentication suite must be one of the following: "
             "Basic, TLS1.2"
         )
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             regex,
             c._set_auth_suite,
@@ -588,7 +658,7 @@ class TestKmipServerConfig(testtools.TestCase):
         # Test that a ConfigurationError is generated when setting the wrong
         # value.
         args = ('invalid',)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             "The flag enabling the TLS certificate client auth flag check "
             "must be a boolean.",
@@ -670,7 +740,7 @@ class TestKmipServerConfig(testtools.TestCase):
         c._logger = mock.MagicMock()
 
         args = (1,)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             "The TLS cipher suites must be a set of strings representing "
             "cipher suite names.",
@@ -687,7 +757,7 @@ class TestKmipServerConfig(testtools.TestCase):
         c._logger = mock.MagicMock()
 
         args = ([0],)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             "The TLS cipher suites must be a set of strings representing "
             "cipher suite names.",
@@ -740,7 +810,7 @@ class TestKmipServerConfig(testtools.TestCase):
         c._logger = mock.MagicMock()
 
         args = (0,)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             "The logging level must be a string representing a valid logging "
             "level.",
@@ -749,10 +819,62 @@ class TestKmipServerConfig(testtools.TestCase):
         )
 
         args = ('invalid',)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             exceptions.ConfigurationError,
             "The logging level must be a string representing a valid logging "
             "level.",
             c._set_logging_level,
             *args
         )
+
+    def test_set_database_path(self):
+        """
+        Test that the database_path configuration property can be set
+        correctly.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        self.assertNotIn('database_path', c.settings.keys())
+
+        with mock.patch('os.path.exists') as os_mock:
+            os_mock.return_value = True
+            c._set_database_path('/test/path/database.db')
+
+        self.assertIn('database_path', c.settings.keys())
+        self.assertEqual(
+            '/test/path/database.db',
+            c.settings.get('database_path')
+        )
+
+    def test_set_database_path_default(self):
+        """
+        Test that the database_path configuration property can be set correctly
+        without specifying a value.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        self.assertNotIn('database_path', c.settings.keys())
+
+        c._set_database_path(None)
+        self.assertIn('database_path', c.settings.keys())
+        self.assertEqual(None, c.settings.get('database_path'))
+
+    def test_set_database_path_invalid_value(self):
+        """
+        Test that the right error is raised when an invalid value is used to
+        set the database_path configuration property.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        self.assertNotIn('database_path', c.settings.keys())
+
+        args = (1, )
+        self.assertRaises(
+            exceptions.ConfigurationError,
+            c._set_database_path,
+            *args
+        )
+        self.assertNotEqual(1, c.settings.get('database_path'))
