@@ -45,14 +45,14 @@ from kmip.core.messages.contents import ProtocolVersion
 from kmip.core.messages import payloads
 
 from kmip.core.misc import Offset
-from kmip.core.misc import QueryFunction
 from kmip.core.misc import ServerInformation
-from kmip.core.misc import VendorIdentification
 
+from kmip.core import objects
 from kmip.core.objects import TemplateAttribute
 from kmip.core.objects import CommonTemplateAttribute
 from kmip.core.objects import PrivateKeyTemplateAttribute
 from kmip.core.objects import PublicKeyTemplateAttribute
+from kmip.core import primitives
 
 from kmip.services.kmip_client import KMIPProxy
 
@@ -64,8 +64,6 @@ from kmip.services.results import OperationResult
 from kmip.services.results import QueryResult
 from kmip.services.results import RekeyKeyPairResult
 
-import kmip.core.utils as utils
-
 import mock
 import os
 import socket
@@ -96,6 +94,62 @@ class TestKMIPClient(TestCase):
     def tearDown(self):
         super(TestKMIPClient, self).tearDown()
 
+    def test_kmip_version_get(self):
+        """
+        Test that the KMIP version can be obtained from the client.
+        """
+        client = KMIPProxy()
+        self.assertEqual(client.kmip_version, enums.KMIPVersion.KMIP_1_2)
+
+    def test_kmip_version_set(self):
+        """
+        Test that the KMIP version of the client can be set to a new value.
+        """
+        client = KMIPProxy()
+        self.assertEqual(client.kmip_version, enums.KMIPVersion.KMIP_1_2)
+        client.kmip_version = enums.KMIPVersion.KMIP_1_1
+        self.assertEqual(client.kmip_version, enums.KMIPVersion.KMIP_1_1)
+
+    def test_kmip_version_set_error(self):
+        """
+        Test that the right error gets raised when setting the client KMIP
+        version with an invalid value.
+        """
+        client = KMIPProxy()
+        args = (client, "kmip_version", None)
+        self.assertRaisesRegex(
+            ValueError,
+            "KMIP version must be a KMIPVersion enumeration",
+            setattr,
+            *args
+        )
+
+    def test_init_with_invalid_config_file_value(self):
+        """
+        Test that the right error is raised when an invalid configuration file
+        value is provided to the client.
+        """
+        kwargs = {'config_file': 1}
+        self.assertRaisesRegex(
+            ValueError,
+            "The client configuration file argument must be a string.",
+            KMIPProxy,
+            **kwargs
+        )
+
+    def test_init_with_invalid_config_file_path(self):
+        """
+        Test that the right error is raised when an invalid configuration file
+        path is provided to the client.
+        """
+        kwargs = {'config_file': 'invalid'}
+        self.assertRaisesRegex(
+            ValueError,
+            "The client configuration file 'invalid' does not exist.",
+            KMIPProxy,
+            **kwargs
+        )
+
     def test_close(self):
         """
         Test that calling close on the client works as expected.
@@ -144,31 +198,17 @@ class TestKMIPClient(TestCase):
     def test_build_credential(self):
         username = 'username'
         password = 'password'
-        cred_type = CredentialType.USERNAME_AND_PASSWORD
         self.client.username = username
         self.client.password = password
 
         credential = self.client._build_credential()
 
-        message = utils.build_er_error(credential.__class__, 'type',
-                                       cred_type,
-                                       credential.credential_type.value,
-                                       'value')
-        self.assertEqual(CredentialType.USERNAME_AND_PASSWORD,
-                         credential.credential_type.value,
-                         message)
-
-        message = utils.build_er_error(
-            credential.__class__, 'type', username,
-            credential.credential_value.username.value, 'value')
-        self.assertEqual(username, credential.credential_value.username.value,
-                         message)
-
-        message = utils.build_er_error(
-            credential.__class__, 'type', password,
-            credential.credential_value.password.value, 'value')
-        self.assertEqual(password, credential.credential_value.password.value,
-                         message)
+        self.assertEqual(
+            CredentialType.USERNAME_AND_PASSWORD,
+            credential.credential_type
+        )
+        self.assertEqual(username, credential.credential_value.username)
+        self.assertEqual(password, credential.credential_value.password)
 
     def test_build_credential_no_username(self):
         username = None
@@ -333,9 +373,6 @@ class TestKMIPClient(TestCase):
 
         payload = batch_item.request_payload
 
-        if query_functions is None:
-            query_functions = list()
-
         msg = base.format(payloads.QueryRequestPayload, payload)
         self.assertIsInstance(payload, payloads.QueryRequestPayload, msg)
 
@@ -344,7 +381,8 @@ class TestKMIPClient(TestCase):
 
     def test_build_query_batch_item_with_input(self):
         self._test_build_query_batch_item(
-            [QueryFunction(QueryFunctionEnum.QUERY_OBJECTS)])
+            [QueryFunctionEnum.QUERY_OBJECTS]
+        )
 
     def test_build_query_batch_item_without_input(self):
         self._test_build_query_batch_item(None)
@@ -385,7 +423,7 @@ class TestKMIPClient(TestCase):
         self.assertEqual(protocol_versions, observed, msg)
 
     def test_build_discover_versions_batch_item_with_input(self):
-        protocol_versions = [ProtocolVersion.create(1, 0)]
+        protocol_versions = [ProtocolVersion(1, 0)]
         self._test_build_discover_versions_batch_item(protocol_versions)
 
     def test_build_discover_versions_batch_item_no_input(self):
@@ -498,9 +536,12 @@ class TestKMIPClient(TestCase):
         msg = base.format(expected, observed)
         self.assertEqual(expected, observed, msg)
 
-        self.assertRaisesRegexp(ValueError, "no processor for operation",
-                                self.client._get_batch_item_processor,
-                                0xA5A5A5A5)
+        self.assertRaisesRegex(
+            ValueError,
+            "no processor for operation",
+            self.client._get_batch_item_processor,
+            0xA5A5A5A5
+        )
 
         expected = self.client._process_get_attributes_batch_item
         observed = self.client._get_batch_item_processor(
@@ -585,7 +626,7 @@ class TestKMIPClient(TestCase):
         self._test_process_query_batch_item(
             list(),
             list(),
-            VendorIdentification(),
+            "",
             ServerInformation(),
             list(),
             list())
@@ -612,7 +653,7 @@ class TestKMIPClient(TestCase):
         self.assertEqual(protocol_versions, result.protocol_versions, msg)
 
     def test_process_discover_versions_batch_item_with_results(self):
-        protocol_versions = [ProtocolVersion.create(1, 0)]
+        protocol_versions = [ProtocolVersion(1, 0)]
         self._test_process_discover_versions_batch_item(protocol_versions)
 
     def test_process_discover_versions_batch_item_no_results(self):
@@ -660,13 +701,21 @@ class TestKMIPClient(TestCase):
         host_list_string = '127.0.0.1,127.0.0.3,  127.0.0.5'
         host_list_expected = ['127.0.0.1', '127.0.0.3', '127.0.0.5']
 
-        self.client._set_variables(host=host_list_string,
-                                   port=None, keyfile=None, certfile=None,
-                                   cert_reqs=None, ssl_version=None,
-                                   ca_certs=None,
-                                   do_handshake_on_connect=False,
-                                   suppress_ragged_eofs=None, username=None,
-                                   password=None, timeout=None)
+        self.client._set_variables(
+            host=host_list_string,
+            port=None,
+            keyfile=None,
+            certfile=None,
+            cert_reqs=None,
+            ssl_version=None,
+            ca_certs=None,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=None,
+            username=None,
+            password=None,
+            timeout=None,
+            config_file=None
+        )
         self.assertEqual(host_list_expected, self.client.host_list)
 
     def test_host_is_invalid_input(self):
@@ -729,6 +778,205 @@ class TestKMIPClient(TestCase):
     @mock.patch(
         'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
     )
+    def test_check(self, send_mock, build_mock):
+        """
+        Test that the client can correctly build, send, and process a Check
+        request.
+        """
+        payload = payloads.CheckResponsePayload(
+            unique_identifier='1',
+            usage_limits_count=100,
+            cryptographic_usage_mask=12,
+            lease_time=10000
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.CHECK),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.check(
+            '1',
+            100,
+            [
+                enums.CryptographicUsageMask.ENCRYPT,
+                enums.CryptographicUsageMask.DECRYPT
+            ],
+            10000
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(100, result.get('usage_limits_count'))
+        self.assertEqual(
+            [
+                enums.CryptographicUsageMask.ENCRYPT,
+                enums.CryptographicUsageMask.DECRYPT
+            ],
+            result.get('cryptographic_usage_mask')
+        )
+        self.assertEqual(10000, result.get('lease_time'))
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._build_request_message'
+    )
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_rekey(self, send_mock, build_mock):
+        """
+        Test that the client can correctly build, send, and process a Rekey
+        request.
+        """
+        payload = payloads.RekeyResponsePayload(
+            unique_identifier='1',
+            template_attribute=objects.TemplateAttribute(
+                attributes=[
+                    objects.Attribute(
+                        attribute_name=objects.Attribute.AttributeName(
+                            'Cryptographic Algorithm'
+                        ),
+                        attribute_value=primitives.Enumeration(
+                            enums.CryptographicAlgorithm,
+                            value=enums.CryptographicAlgorithm.AES,
+                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
+                        )
+                    ),
+                    objects.Attribute(
+                        attribute_name=objects.Attribute.AttributeName(
+                            'Cryptographic Length'
+                        ),
+                        attribute_value=primitives.Integer(
+                            value=128,
+                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
+                        )
+                    )
+                ]
+            )
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.REKEY),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.rekey(
+            uuid='1',
+            offset=0,
+            template_attribute=objects.TemplateAttribute(
+                attributes=[
+                    objects.Attribute(
+                        attribute_name=objects.Attribute.AttributeName(
+                            'Activation Date'
+                        ),
+                        attribute_value=primitives.DateTime(
+                            value=1136113200,
+                            tag=enums.Tags.ACTIVATION_DATE
+                        )
+                    )
+                ]
+            )
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(
+            objects.TemplateAttribute(
+                attributes=[
+                    objects.Attribute(
+                        attribute_name=objects.Attribute.AttributeName(
+                            'Cryptographic Algorithm'
+                        ),
+                        attribute_value=primitives.Enumeration(
+                            enums.CryptographicAlgorithm,
+                            value=enums.CryptographicAlgorithm.AES,
+                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
+                        )
+                    ),
+                    objects.Attribute(
+                        attribute_name=objects.Attribute.AttributeName(
+                            'Cryptographic Length'
+                        ),
+                        attribute_value=primitives.Integer(
+                            value=128,
+                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
+                        )
+                    )
+                ]
+            ),
+            result.get('template_attribute')
+        )
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch('kmip.services.kmip_client.KMIPProxy._build_request_message')
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_rekey_with_no_payload(self, send_mock, build_mock):
+        """
+        Test that the client correctly handles responses with no payload data.
+        """
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.REKEY),
+            result_status=ResultStatus(ResultStatusEnum.OPERATION_FAILED),
+            result_reason=ResultReason(ResultReasonEnum.PERMISSION_DENIED),
+            result_message=ResultMessage("Permission denied."),
+            response_payload=None
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.rekey(
+            uuid='1',
+            offset=0,
+            template_attribute=objects.TemplateAttribute(
+                attributes=[
+                    objects.Attribute(
+                        attribute_name=objects.Attribute.AttributeName(
+                            'Activation Date'
+                        ),
+                        attribute_value=primitives.DateTime(
+                            value=1136113200,
+                            tag=enums.Tags.ACTIVATION_DATE
+                        )
+                    )
+                ]
+            )
+        )
+
+        self.assertEqual(
+            ResultStatusEnum.OPERATION_FAILED,
+            result.get('result_status')
+        )
+        self.assertEqual(
+            ResultReasonEnum.PERMISSION_DENIED,
+            result.get('result_reason')
+        )
+        self.assertEqual("Permission denied.", result.get('result_message'))
+
+    @mock.patch('kmip.services.kmip_client.KMIPProxy._build_request_message')
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
     def test_derive_key(self, send_mock, build_mock):
         """
         Test that the client can derive a key.
